<?php

/**
 * Twilio SMS Notifications Add-on for Sprout Invoices controller.
 *
 * @package Sprout_Invoices
 * @subpackage Twilio Notifications
 */
class SI_Twilio_Notifications extends SI_Controller {

	const TWIL_ENABLED      = 'si_twilio_enabled';
	const TWIL_API_KEY      = 'si_twilio_api_key_field';
	const TWIL_AUTH_KEY     = 'si_twilio_auth_key_field';
	const TWIL_SENDER_NO    = 'si_twilio_send_number_field';
	const TWIL_MESSAGE_TMPL = 'si_twilio_message_field';

	private static $twilio_enabled;
	private static $twilio_api_key;
	private static $twilio_auth_key;
	private static $twilio_number;
	private static $twilio_template;

	/**
	 * Addon init function.
	 */
	public static function init() {
		self::$twilio_enabled  = get_option( self::TWIL_ENABLED, false );
		self::$twilio_api_key  = get_option( self::TWIL_API_KEY, 'empty' );
		self::$twilio_auth_key = get_option( self::TWIL_AUTH_KEY, 'empty' );
		self::$twilio_number   = get_option( self::TWIL_SENDER_NO, 'empty' );
		self::$twilio_template = get_option( self::TWIL_MESSAGE_TMPL, 'Hello, Your invoice {title} at {sitename} is now {status}, You can view it here {link} . Thanks for your custom.' );

		add_filter( 'si_settings', array( __CLASS__, 'register_settings' ) );

		add_action( 'admin_init', array( __CLASS__, 'register_twilio_meta_boxes' ) );

		add_action( 'admin_init', array( __CLASS__, 'register_resources' ) );

		add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_enqueue' ), 20 );

		// AJAX action to handle test request
		add_filter( 'wp_ajax_sa_send_twilio_notification', array( __CLASS__, 'maybe_send_twilio_notification' ) );
	}


	/**
	 * Register twilio JS.
	 *
	 * @since 20.5.2
	 */
	public static function register_resources() {
		wp_register_script(
			'si_twilio',
			SA_ADDON_TWILIO_NOTIFICATIONS_URL . '/resources/si-twilio-jquery.js',
			array( 'jquery', 'qtip' ),
			self::SI_VERSION,
			true
		);
	}

	/**
	 * Enqueue twilio JS.
	 *
	 * @since 20.5.2
	 */
	public static function admin_enqueue() {
		wp_enqueue_script( 'si_twilio' );
	}

	/**
	 * Defining Settings for Twilio Integration
	 *
	 * @param  array $settings array of the settings.
	 * @return array $settings array of the settings.
	 */
	public static function register_settings( $settings = array() ) {
		// Settings
		$settings['sprout-invoices-add-on-twilio-sms-notifications'] = array(
			'title'       => __( 'Twilio Notifications', 'sprout-invoices' ),
			'description' => __( 'Please enter your API Information for your Twilio Account. These Notifications are sent automatically with Email Notifications', 'sprout-invoices' ),
			'weight'      => 1020, // Add-on settings are 1000 plus
			'tab'         => '52', // Settings ID
			'callback'    => array( __CLASS__, 'display_twilio_sms_notification_message' ),
			'settings'    => array(
				self::TWIL_ENABLED => array(
					'label'  => __( 'Send SMS Notifications', 'sprout-invoices' ),
					'option' => array(
						'type'        => 'checkbox',
						'value'       => 'archived',
						'attributes'  => array(),
						'default'     => self::$twilio_enabled,
						'description' => __( 'Turn On/Off SMS Notifications', 'sprout-invoices' ),
					),
				),
				self::TWIL_API_KEY => array(
					'label'  => __( 'Twilio API Key (Account SID)', 'sprout-invoices' ),
					'option' => array(
						'type'        => 'text',
						'attributes'  => array(),
						'default'     => self::$twilio_api_key,
						'description' => __( 'Your Twilio API Key (Account SID)', 'sprout-invoices' ),
					),
				),
				self::TWIL_AUTH_KEY => array(
					'label'  => __( 'Twilio Auth Token', 'sprout-invoices' ),
					'option' => array(
						'type'        => 'text',
						'attributes'  => array(),
						'default'     => self::$twilio_auth_key,
						'description' => __( 'Your Twilio Auth Token', 'sprout-invoices' ),
					),
				),
				self::TWIL_SENDER_NO => array(
					'label'  => __( 'Twilio Sender Number', 'sprout-invoices' ),
					'option' => array(
						'type'        => 'text',
						'attributes'  => array(),
						'default'     => self::$twilio_number,
						'description' => __( 'Your Twilio Send Number', 'sprout-invoices' ),
					),
				),
				self::TWIL_MESSAGE_TMPL => array(
					'label'  => __( 'Your message to Send', 'sprout-invoices' ),
					'option' => array(
						'type'        => 'textarea',
						'attributes'  => array(),
						'default'     => self::$twilio_template,
						'description' => __( 'The Message you would like your users to recieve.', 'sprout-invoices' )  . self::display_dynamic_tags(),
					),
				),
			),
		);
		return $settings;
	}

	/**
	 * Register twilio meta boxes.
	 */
	public static function register_twilio_meta_boxes() {
		$args = array(
			'si_twilio_send' => array(
				'title'         => __( 'Twilio Notification', 'sprout-invoices' ),
				'show_callback' => array( __CLASS__, 'show_twilio_meta_box' ),
				'context'       => 'normal',
				'priority'      => 'high',
				'save_priority' => 500,
				'weight'        => 10,
			),
		);
		do_action( 'sprout_meta_box', $args, SI_Invoice::POST_TYPE );
	}

	/**
	 * Display the twilio meta box.
	 *
	 * @param object $post post object.
	 * @param array  $metabox array of the meta box.
	 * @return string
	 */
	public static function show_twilio_meta_box( $post, $metabox ) {
		$invoice_id = $post->ID;
		$invoice    = SI_Invoice::get_instance( $invoice_id );
		if ( $post->post_status == 'auto-draft' ) {
			printf( '<p>%s</p>', esc_html__( 'Save Invoice Before Sending.', 'sprout-invoices' ) );
			return;
		} elseif ( ! $invoice->get_client_id() ) {
			printf( '<p>%s</p>', esc_html__( 'Select a client and save.', 'sprout-invoices' ) );
			return;
		} else {
			$fields = self::meta_box_option_fields( $invoice );
			self::load_addon_view(
				'Twilio_Admin.php',
				array(
					'fields'     => $fields,
					'invoice_id' => $invoice_id,
				),
				false
			);
		}
	}

	/**
	 * Twilio SMS Notification option fields.
	 *
	 * @param  object $invoice invoice object.
	 * @return array  $fields  array of fields.
	 */
	public static function meta_box_option_fields( $invoice ) {
		$invoice_id = $invoice->get_id();
		$client_id  = $invoice->get_client_id();
		$fields     = array();
		if ( $client_id ) {
			$client_phone = si_get_client_phone();

			$fields['SI_Twillio_Client_Tag']   = array(
				'type'   => 'bypass',
				'weight' => 20,
				'output' => '<hr/>',
			);
			$fields['SI_Twillio_Client_Phone'] = array(
				'type'   => 'bypass',
				'weight' => 30,
				'title'  => __( 'Client Phone Number', 'sprout-invoices' ),
				'output' => sprintf( 'Client Phone Number: %s', $client_phone, 'sprout-invoices' ),
			);
			// filter and sort
			uasort( $fields, array( __CLASS__, 'sort_by_weight' ) );
		}
		return $fields;
	}

	/**
	 * Maybe send the SMS notification.
	 */
	public static function maybe_send_twilio_notification() {
		self::send_twilio_notification();
	}

	/**
	 * Display the SMS notification.
	 */
	public static function display_twilio_sms_notification_message() {
		$sid   = self::$twilio_api_key;
		$token = self::$twilio_auth_key;

		printf(
			'<p>%s</p>',
			esc_html__( 'Set your Twilio API keys here. Make sure to read the description of each setting/option before saving.', 'sprout-invoices' )
		);
	}

	/**
	 * Display the dynamic tags for the notification message.
	 *
	 * @param  string $context context of the notification.
	 * @return string $tags dynamic tags.
	 */
	public static function display_dynamic_tags( $context = '' ) {
		if ( '' === $context ) {
			$context = __( 'Invoice', 'sprout-invoices' );
		}
		$tags = '';
		foreach ( self::get_registered_tags() as $tag => $description ) {
			$tags .= sprintf( '<p><b>{%s}</b>&nbsp;&mdash;&nbsp;%s</p>', $tag, $description );
		}
		return $tags;
	}

	/**
	 * Get the registered tags.
	 *
	 * @return array $tags array of tags.
	 */
	public static function get_registered_tags() {
		$tags             = array();
		$tags['sitename'] = esc_html__( 'Your Site Name', 'sprout-invoices' );
		$tags['status']   = esc_html__( 'The Invoice Status', 'sprout-invoices' );
		$tags['link']     = esc_html__( 'The URL To the Invoice', 'sprout-invoices' );
		$tags['title']    = esc_html__( 'The title of the invoice', 'sprout-invoices' );
		return apply_filters( 'si_twilio_notifications_formatting_tags', $tags );
	}

	/**
	 * Send the SMS notification.
	 *
	 * @return bool
	 */
	public static function send_twilio_notification() {
		$post_id = intval( $_POST['post_id'] );
		if ( self::$twilio_enabled ) {
			try {
				$invoice_id = $post_id;
				$invoice    = SI_Invoice::get_instance( $invoice_id );

				// Create Twilio Instance.
				$sid    = self::$twilio_api_key;
				$token  = self::$twilio_auth_key;
				$twilio = new Twilio\Rest\Client( $sid, $token );

				// Get Client Phone Number.
				$client   = $invoice->get_client();
				$clientNo = $client->get_phone();
				if ( ! empty( $clientNo ) ) {
					$twilio->messages->create(
						$clientNo,
						array(
							'from' => self::$twilio_number,
							'body' => self::generate_sms_message( $invoice ),
						)
					);
					echo sprintf( '<p>%s</p>', esc_html__( 'Your message has been sent, The message body was: ' . self::generate_sms_message( $invoice ), 'sprout-invoices' ) );
					die;
				}
			} catch ( Exception $e ) {
				self::ajax_fail( 'Error Processing Text Message.' );
				do_action( 'si_error', 'Send Twilio Error', $e->getMessage() );
				return;
			}
		} else {
			self::ajax_fail( 'Twilio is not enabled.' );
			return;
		}
	}

	/**
	 * Get SMS status message.
	 *
	 * @param  string $status Status of the SMS notification.
	 */
	public static function get_status_sms_string( $status ) {
		switch ( $status ) {
			case 'ending':
				return 'Pending Payment';
			break;
			case 'complete':
				return 'Paid in Full';
			break;
			case 'publish':
				return 'Available';
			break;
			default:
				return ucfirst( $status );
		}
	}
	/**
	 * Generate the SMS message.
	 *
	 * @param  SI_Invoice $invoice Invoice object.
	 * @return string $message SMS message.
	 */
	public static function generate_sms_message( $invoice ) {
		// Get Invoice Details.
		$id     = $invoice->get_invoice_id();
		$status = self::get_status_sms_string( $invoice->get_status() );
		$link   = get_the_permalink( $id );
		$title  = get_the_title( $id );

		$message = str_replace(
			array(
				'{sitename}',
				'{status}',
				'{link}',
				'{title}',
			),
			array(
				get_bloginfo( 'name' ),
				$status,
				$link,
				$title,
			),
			self::$twilio_template
		);
		return $message;
	}

	/**
	 * Load addon view.
	 *
	 * @param  string $view view name.
	 * @param  array  $args data to pass to view.
	 * @param  bool   $allow_theme_override option to overide the sprout invoices theme.
	 * @return string $view HTML of the view.
	 */
	public static function load_addon_view( $view, $args, $allow_theme_override = true ) {
		add_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		$view = self::load_view( $view, $args, $allow_theme_override );
		remove_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		return $view;
	}
	/**
	 * Load addon view to string.
	 *
	 * @param  string $view view name.
	 * @param  array  $args data to pass to view.
	 * @param  bool   $allow_theme_override option to overide the sprout invoices theme.
	 * @return string $view HTML of the view.
	 */
	public static function load_addon_view_to_string( $view, $args, $allow_theme_override = true ) {
		ob_start();
		self::load_addon_view( $view, $args, $allow_theme_override );
		return ob_get_clean();
	}

	/**
	 * Get the path to the addons views.
	 *
	 * @return string
	 */
	public static function addons_view_path() {
		return SA_ADDON_TWILIO_NOTIFICATIONS_PATH . '/views/';
	}

}
