<?php
/**
 * Sprout invoices - WooCommerce Checkout Payment Processor.
 *
 * This adds a WooCommerce Checkout payment processor to Sprout Invoices. This class takes the invoices line items and adds them to the woocommerce cart and redirects to the checkout page.
 * If a Woocommerce product is found in the line items, it will link it with that product in Woocommerce. For generic line items it will create a puesdo product called Invoice Payment.
 *
 * @since 13.0.3
 */
class SI_Woo_Payment_Processor extends SI_Offsite_Processors {
	const PAYMENT_METHOD           = 'WooCommerce Checkout';
	const PAYMENT_SLUG             = 'woocheckout';
	const WOO_PRODUCT_ID_OPTION    = 'woocheckout_product';
	const SI_WOO_INVOICE_ID_OPTION = 'si_woo_invoice_id';
	/**
	 * Instance of this class.
	 *
	 * @var object
	 */
	protected static $instance;

	/**
	 * Get WooCommerce Checkout Payment Processor Instance.
	 *
	 * @since 13.0.3
	 * @return Object $instance SI_Woo_Payment_Processor
	 */
	public static function get_instance() {
		if ( ! ( isset( self::$instance ) && is_a( self::$instance, __CLASS__ ) ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Get the name of the payment method.
	 *
	 * @since 13.0.3
	 * @return string Payment method name.
	 */
	public function get_payment_method() {
		return self::PAYMENT_METHOD;
	}

	/**
	 * Get the slug of the payment method.
	 *
	 * @since 13.0.3
	 * @return string Payment method slug.
	 */
	public function get_slug() {
		return self::PAYMENT_SLUG;
	}

	/**
	 * Register the payment processor and necessary hooks.
	 *
	 * @since 13.0.3
	 * @return mixed Return early if WooCommerce is not active.
	 */
	public static function register() {

		if ( ! function_exists( 'wc_attribute_taxonomy_name' ) ) {
			return;
		}

		self::add_payment_processor( __CLASS__, __( 'WooCommerce Checkout', 'sprout-invoices' ) );

		add_filter( 'woocommerce_available_payment_gateways', array( __CLASS__, 'maybe_remove_si_processor' ) );

		if ( self::is_processor_enabled( __CLASS__ ) ) {
			add_action( 'admin_init', array( __CLASS__, 'maybe_create_woo_product' ) );
		}

		if ( self::is_processor_enabled( __CLASS__ ) ) {
			add_action( 'woocommerce_thankyou', array( __CLASS__, 'redirect_to_invoice_after_payment' ) );
		}
	}

	/**
	 * Get public name used in the invoice payment dropdown.
	 *
	 * @since 13.0.3
	 * @return string Public name.
	 */
	public static function public_name() {
		return __( 'WooCommerce Checkout', 'sprout-invoices' );
	}

	/**
	 * Get the checkout options.
	 *
	 * @since 13.0.3
	 * @return array $option Checkout options.
	 */
	public static function checkout_options() {
		$option = array(
			'label'          => esc_html__( 'WooCommerce Checkout', 'sprout-invoices' ),
			'accepted_cards' => array(),
		);
		return apply_filters( 'si_woo_checkout_options', $option );
	}

	/**
	 * Contructor for class.
	 *
	 * @since 13.0.3
	 * Adds checkout action hook to redirect to WooCommerce checkout and removes sprout invoices checkout page.
	 */
	protected function __construct() {
		parent::__construct();

		// Remove pages
		add_filter( 'si_checkout_pages', array( $this, 'remove_checkout_pages' ) );
		add_action( 'si_checkout_action_' . SI_Checkouts::PAYMENT_PAGE, array( $this, 'send_woo_checkout' ), 0, 1 );

		// Woocommerce hooks
		add_filter( 'woocommerce_calculated_total', array( $this, 'maybe_change_total' ), 10, 2 );
		add_action( 'woocommerce_review_order_before_order_total', array( $this, 'edit_checkout_details' ) );
		add_action( 'woocommerce_checkout_create_order', array( $this, 'update_order_details' ), 20, 1 );
	}

	/**
	 * Remove SI invoice payment gateway if using WooCommerce checkout.
	 *
	 * @since 13.0.3
	 * @param  array $available_gateways Array of available gateways in Woocommerce.
	 * @return array
	 */
	public static function maybe_remove_si_processor( $available_gateways ) {
		if ( isset( WC()->session ) ) {
			if ( ! is_null( WC()->session->get( 'invoice_id' ) ) ) {
				unset( $available_gateways['sprout_invoices'] );
			}
		}
		return $available_gateways;
	}

	/**
	 * Create a Woocommerce product from Sprout Invoices.
	 *
	 * @since 13.0.3
	 * @return string $product_id
	 */
	public static function maybe_create_woo_product() {
		$product_id = self::get_woo_product_id( false, true );

		if ( ! function_exists( 'wc_attribute_taxonomy_name' ) ) {
			return;
		}

		if ( ! taxonomy_exists( wc_attribute_taxonomy_name( 'si-invoice-id' ) ) ) {
			global $wpdb;

			$attribute_name  = sanitize_title( esc_attr( 'si-invoice-id' ) );
			$attribute_type  = esc_attr( 'text' );
			$attribute_label = esc_attr( 'Invoice ID' );

			$wpdb->insert(
				$wpdb->prefix . 'woocommerce_attribute_taxonomies',
				array(
					'attribute_name'  => $attribute_name,
					'attribute_label' => $attribute_label,
					'attribute_type'  => $attribute_type,
				)
			);

			delete_transient( 'wc_attribute_taxonomies' );
		}

		return $product_id;
	}

	/**
	 * Get the Woocommerce product ID.
	 *
	 * @since 13.0.3
	 * @param boolean $deep_search If true then search for the product in all Woocommerce products.
	 * @param boolean $create If true then create the product if it doesn't exist.
	 * @return string $product_id
	 */
	public static function get_woo_product_id( $deep_search = false, $create = false ) {
		$_product_id = get_option( self::WOO_PRODUCT_ID_OPTION, false );
		if ( 'product' === get_post_type( $_product_id ) ) {
			return $_product_id;
		}
		$product_id = 0;
		if ( ! $product_id && $deep_search ) {
			// query woo products
			$args     = array(
				'post_type'      => 'product',
				'post_status'    => 'publish',
				'posts_per_page' => -1,
				'fields'         => 'ids',
				'meta_query'     => array(
					array(
						'key'     => self::WOO_PRODUCT_ID_OPTION,
						'compare' => 'EXISTS',
					),
				),
			);
			$products = get_posts( apply_filters( 'si_get_woo_product_id', $args ) );

			if ( ! empty( $products ) ) {
				$product_id = array_pop( $products );
			}
		}
		if ( ! $product_id && $create ) {
			$product_id = self::create_woo_product_for_checkout();
		}
		update_option( self::WOO_PRODUCT_ID_OPTION, $product_id );
		return $product_id;
	}

	/**
	 * Create product for Sprout Invoice line items that are not Woocommerce Products.
	 *
	 * @since 13.0.3
	 * @param array $item The line item.
	 * @return int  $post_id The post ID of the product.
	 */
	public static function create_woo_product_for_checkout( $item = array() ) {
		$woo_product = new WC_Product_Simple();
		$woo_product->set_name( wp_strip_all_tags( $item['desc'] ) );
		$woo_product->set_status( 'publish' );
		$woo_product->set_catalog_visibility( 'hidden' );
		$woo_product->set_description( wp_strip_all_tags( $item['desc'] ) );

		// If there is a deposit then set the price to the deposit amount, else set it to the total (inclusive of taxes, discouts, etc.)
		if ( isset( $item['deposit'] ) ) {
			$woo_product->set_regular_price( $item['rate'] );
		} else {
			/**
			 * If woocommerce taxes are enabled then set the price to the rate so that woocommerce can calculate tax
			 * else set it to the total which is inclusive of tax.
			 */
			if ( 'yes' === get_option( 'woocommerce_calc_taxes' ) ) {
				$woo_product->set_regular_price( $item['rate'] );
			} else {
				$woo_product->set_regular_price( $item['total'] );
			}
		}

		if ( isset( $item['deposit'] ) ) {
			$woo_product->update_meta_data( 'deposit', $item['deposit'] );
		} else {
			$woo_product->update_meta_data( 'line_item', 1 ); // set this to 1 so that we can delete it later after the order is complete.
		}
		$woo_product_id = $woo_product->save();
		return $woo_product_id;
	}

	/**
	 * Update the Invoice Payment Product for each line item that is not a Woocommerce Product.
	 *
	 * This will also update for every new invoice as well.
	 *
	 * @since 13.0.3
	 * @deprecated deprecated in 20.4 in favor of adding each line item as products.
	 * @param SI_Invoice $invoice Sprout Invoices invoice Object.
	 * @param string     $product_id The product ID of the product to update.
	 *
	 * @return string $variant_id
	 */
	public static function add_product_variation_for_invoice_payment( SI_Invoice $invoice, $product_id = 0 ) {
		_deprecated_function( __METHOD__, '20.4' );

		if ( ! $product_id ) {
			$product_id = self::get_woo_product_id( true, true );
		}

		$invoice_id     = $invoice->get_id();
		$payment_amount = ( si_has_invoice_deposit( $invoice_id ) ) ? $invoice->get_deposit() : $invoice->get_balance();
		$attribute_name = 'pa_si-invoice-id';

		// update original product.
		wp_set_object_terms( $product_id, array( $invoice_id ), $attribute_name );

		$product_attributes_data                    = array();
		$product_attributes_data[ $attribute_name ] = array(
			'name'         => $attribute_name,
			'value'        => '',
			'is_visible'   => '1',
			'is_variation' => '1',
			'is_taxonomy'  => '1',
		);

		// Attach the above array to the new posts meta data key '_product_attributes'.
		update_post_meta( $product_id, '_product_attributes', $product_attributes_data );

		$description = sprintf( 'Invoice <a href="%s">#%s</a>', get_permalink( $invoice_id ), $invoice_id );

		// create variation for Woocommerce.
		$variation_post = array(

			'post_title'   => 'Invoice #' . $invoice_id,
			'post_name'    => 'product-' . $product_id . '-variation-' . $invoice_id,
			'post_content' => $description,
			'post_status'  => 'publish',
			'post_parent'  => $product_id,
			'post_type'    => 'product_variation',
			'guid'         => home_url() . '/?product_variation=product-' . $product_id . '-variation-' . $invoice_id,
		);

		$varient_id = wp_insert_post( $variation_post );

		update_post_meta( $varient_id, '_description', $description ); // added variable description.

		update_post_meta( $varient_id, '_price', (float) $payment_amount );
		update_post_meta( $varient_id, '_regular_price', (float) $payment_amount );
		update_post_meta( $varient_id, '_tax_class', apply_filters( 'si_woocommerce_varient_tax_rate', 'zero-rate' ) );
		update_post_meta( $varient_id, '_virtual', 'yes' );
		update_post_meta( $varient_id, '_invoice_id', $invoice_id );

		do_action( 'si_woo_varient_created', $varient_id, $invoice );
		return $varient_id;

	}

	/**
	 * The review page is unnecessary
	 *
	 * @param  array $pages The checkout pages.
	 * @return array $pages
	 */
	public function remove_checkout_pages( $pages ) {
		unset( $pages[ SI_Checkouts::REVIEW_PAGE ] );
		return $pages;
	}

	/**
	 * Create deposit array.
	 *
	 * @since 20.5
	 * @param SI_Invoice $invoice The invoice object.
	 * @param int        $invoice_id The invoice id.
	 * @return array $deposit_item_array The deposit item array.
	 */
	public function create_deposit_array( $invoice, $invoice_id ) {
		/**
		 * If the invoice has a deposit, create a deposit product and
		 * add it to the cart but do not add line items as the customer only needs to pay the deposit.
		 */
		$deposit_item_array = array(
			'qty'     => 1, // Product quantity is always 1 for deposits.
			'rate'    => $invoice->get_deposit(),
			'deposit' => true,
			'desc'    => sprintf(
				// translators: %s: invoice id
				__(
					'Deposit for invoice #%s',
					'sprout-invoices'
				),
				$invoice_id
			),
		);
		return $deposit_item_array;
	}

	/**
	 * Create Partial Payment array.
	 *
	 * @since 20.5
	 * @param SI_Invoice $invoice The invoice object.
	 * @param int        $invoice_id The invoice id.
	 * @return array $partial_payment_array The partial payment array.
	 */
	public function create_partial_payment_array( $invoice, $invoice_id ) {
		/**
		 * If the invoice has a partial payment, create a partial payment product and
		 * add it to the cart but do not add line items as the customer only needs to pay the partial payment.
		 */
		$partial_payment_array = array(
			'qty'     => 1, // Product quantity is always 1 for deposits.
			'rate'    => $invoice->get_deposit(),
			'deposit' => true,
			'desc'    => sprintf(
				// translators: %s: invoice id
				__(
					'Partial Payment for invoice #%s',
					'sprout-invoices'
				),
				$invoice_id
			),
		);
		return $partial_payment_array;
	}

	/**
	 * Instead of redirecting to the Sprout Invoices checkout page,
	 * add the invoice to the cart and redirect to the Woocommerce checkout page.
	 *
	 * @since 13.0.3
	 * @param SI_Checkouts $checkout The checkout object.
	 * @return void
	 */
	public function send_woo_checkout( SI_Checkouts $checkout ) {

		if ( ! is_a( $checkout->get_processor(), __CLASS__ ) ) {
			return;
		}
		$invoice               = $checkout->get_invoice();
		$invoice_id            = $invoice->get_id();
		$line_items            = $invoice->get_line_items();
		$invoice_has_deposit   = $invoice->has_deposit();
		$maybe_paid_deposit    = SI_Payment::maybe_paid_deposit( $invoice );
		if ( class_exists( 'SI_Partial_Payments' ) ) {
			$maybe_partial_payment = SI_Partial_Payments::maybe_partial_payment( $invoice );
		} else {
			$maybe_partial_payment = false;
		}

		// Make sure the cart is always empty before adding items.
		self::remove_line_item_products();
		WC()->cart->empty_cart();

		// set the invoice id in the session.
		WC()->session->set( 'invoice_id', $invoice_id );

		/**
		 * Check if the invoice has a deposit.
		 * We need to check if the invoice has a deposit asscoiated with it. If it does, we need to create a deposit product
		 * that we can send to Woocommerce. If it does not or the deposit value is equal to the invoice balance, we can treat
		 * the invoice as noraml and redirect to the woocommerce with all of the line items.
		 */
		if ( ! $maybe_paid_deposit && $invoice_has_deposit ) {

			$deposit_item_array = self::create_deposit_array( $invoice, $invoice_id );
			$product_id         = self::create_woo_product_for_checkout( $deposit_item_array );
			WC()->cart->add_to_cart( $product_id, $deposit_item_array['qty'] );
			WC()->session->set( 'si_deposit', $invoice_has_deposit );
		} elseif ( $maybe_partial_payment ) {

			$partial_payment_array = self::create_partial_payment_array( $invoice, $invoice_id );
			$product_id            = self::create_woo_product_for_checkout( $partial_payment_array );
			WC()->cart->add_to_cart( $product_id, $partial_payment_array['qty'] );
			WC()->session->set( 'si_deposit', $invoice_has_deposit );
		} else {

			WC()->session->set( 'si_deposit', false );
			// If the invoice has no deposit, add the each line item to the cart.
			foreach ( $line_items as $key => $item ) {
				if ( empty( $item['wc_product'] ) ) {
					// Add line items to cart that are not WooCommerce products.
					$product_id = self::create_woo_product_for_checkout( $item );
					if ( 'yes' === get_option( 'woocommerce_calc_taxes' ) ) {
						WC()->cart->add_to_cart( $product_id, $item['qty'] );
					} else {
						/**
						 * Product quantity is always 1 for Sprout Invoice line items that are not WooCommerce products and
						 * WooCommerce is not calculating tax. Since the product total already includes line item discount, tax, quantity, etc.
						 * We do not need to add the line item to the cart with a quantity greater than 1.
						 */
						WC()->cart->add_to_cart( $product_id, 1 ); // Product quantity is always 1 check comment above.
					}
				} else {
					if ( empty( $item['wc_variation'] ) ) {
						// Add line items to cart that are WooCommerce simple products.
						$product_id = $item['wc_product'];
						$qty        = $item['qty'];
						WC()->cart->add_to_cart( $product_id, $qty );
					} else {
						// Add line items to cart that are WooCommerce variable products.
						$product_id   = $item['wc_product'];
						$variation_id = $item['wc_variation'];
						$variations   = json_decode( $item['wc_variation_attributes'], true );
						$qty          = $item['qty'];
						WC()->cart->add_to_cart( $product_id, $qty, $variation_id, $variations );
					}
				}
			}
		}

		// get the checkout url
		$checkout_url = wc_get_checkout_url();

		// add the WooCommerce memberships filter back.
		add_filter( 'woocommerce_is_purchasable', array( 'Products', 'product_is_purchasable' ) );

		wp_safe_redirect( $checkout_url );
		exit();
	}

	/**
	 * Edit checkout order details
	 *
	 * This function when the order review is displayed on the checkout page. If the invoice has a deposit or partial payment made
	 * we need to subtract the deposit from the cart total so that the customer only pays the remaining balance.
	 *
	 * @since 20.4
	 * @return void
	 */
	public static function edit_checkout_details() {
		if ( WC()->session->get( 'si_deposit' ) ) {
			return;
		}

		$invoice = SI_Invoice::get_instance( WC()->session->get( 'invoice_id' ) );
		/**
		 * This function is called on the checkout page. If the checkout is not a redirect from Sprout Invoices, we can return.
		 */
		if ( ! is_a( $invoice, 'SI_Invoice' ) ) {
			return;
		}
		$paid = $invoice->get_payments_total( false );
		if ( 0 < $paid ) {
			?>
				<tr>
					<th> <?php esc_html_e( 'Deposit', 'sprout-invoices' ); ?> </th>
					<td>
						<span class="woocommerce-Price-amount amount">
								<b>
									<span class="woocommerce-Price-currencySymbol"><?php echo esc_html( get_woocommerce_currency_symbol() ); ?></span>
									<?php
										echo esc_html( $paid );
									?>
								</b>
						</span>
					</td>
				</tr>
			<?php
		}
		$tax = $invoice->get_tax_total();
		if ( 0 < $tax ) {
			?>
				<tr>
					<th> <?php esc_html_e( 'Invoice Total Tax before Discount', 'sprout-invoices' ); ?> </th>
					<td>
						<span class="woocommerce-Price-amount amount">
								<b>
									<span class="woocommerce-Price-currencySymbol"><?php echo esc_html( get_woocommerce_currency_symbol() ); ?></span>
									<?php
										echo esc_html( $tax );
									?>
								</b>
						</span>
					</td>
				</tr>
			<?php
		}
		$discount = $invoice->get_discount_total();
		if ( 0 < $discount ) {
			?>
				<tr>
					<th> <?php esc_html_e( 'Invoice Total Discount', 'sprout-invoices' ); ?> </th>
					<td>
						<span class="woocommerce-Price-amount amount">
								<b>
									<span class="woocommerce-Price-currencySymbol"><?php echo esc_html( get_woocommerce_currency_symbol() ); ?></span>
									<?php
										echo esc_html( $discount );
									?>
								</b>
						</span>
					</td>
				</tr>
			<?php
		}
		$tax2 = $invoice->get_tax2_total( true );
		if ( 0 < $tax2 ) {
			?>
				<tr>
					<th> <?php esc_html_e( 'Invoice Total Tax before Discount', 'sprout-invoices' ); ?> </th>
					<td>
						<span class="woocommerce-Price-amount amount">
								<b>
									<span class="woocommerce-Price-currencySymbol"><?php echo esc_html( get_woocommerce_currency_symbol() ); ?></span>
									<?php
										echo esc_html( $tax2 );
									?>
								</b>
						</span>
					</td>
				</tr>
			<?php
		}
	}

	/**
	 * Update the cart total.
	 *
	 * This function is called when the cart total is calculated.
	 * If the invoice has a deposit or partial payment made we will subtract it from the cart total.
	 * If the invoice has a tax on the entire invoice it will be added to the cart total.
	 * If the invoice has a discount on the entire invoice it will be subtracted from the cart total.
	 *
	 * @since 20.4
	 * @param  float   $total The current cart total.
	 * @param  WC_Cart $cart The cart object.
	 * @return float   Returns the new total if there are previous payments, otherwise returns the original total.
	 */
	public static function maybe_change_total( $total, $cart ) {

		if ( ! WC()->session->get( 'invoice_id' ) ) {
			return $total;
		}

		if ( WC()->session->get( 'si_deposit' ) ) {
			return $total;
		}

		$invoice   = SI_Invoice::get_instance( WC()->session->get( 'invoice_id' ) );
		$new_total = 0;

		// Previous Payments.
		$paid = $invoice->get_payments_total( false );

		// Taxes Before Discount.
		$tax  = $invoice->get_tax_total();
		$tax2 = $invoice->get_tax2_total();

		// Discounts.
		$discount = $invoice->get_discount_total();

		$total = $total - $paid + $tax + $tax2 - $discount;

		return $total;
	}

	/**
	 * Update the order details.
	 *
	 * This function is called when the order is created.
	 * If the invoice has a deposit or partial payment made we need to add as fee.
	 * This will make sure the order details are correct.
	 *
	 * @since 20.4
	 * @param  WC_Order $order The order object.
	 * @return void
	 */
	public static function update_order_details( $order ) {
		if ( ! WC()->session->get( 'invoice_id' ) ) {
			return;
		}

		if ( WC()->session->get( 'si_deposit' ) ) {
			return;
		}

		$total   = $order->get_total();
		$invoice = SI_Invoice::get_instance( WC()->session->get( 'invoice_id' ) );
		$fees    = Woo_Tools::create_fees( $invoice );

		foreach ( $fees as $item ) {
			$fee = new WC_Order_Item_Fee();
			$fee->set_name( $item['name'] );
			$fee->set_total( $item['amount'] );
			$order->add_item( $fee );
			$order->save();
		}
	}

	/**
	 * Process a payment. This is an abstract method that the gateway must be set even if it is not used.
	 *
	 * @param  SI_Checkouts $checkout The checkout object.
	 * @param  SI_Invoice   $invoice The invoice object.
	 * @return void
	 */
	public function process_payment( SI_Checkouts $checkout, SI_Invoice $invoice ) {
		// Nothing, since the user is not returned.
	}

	/**
	 * Create a Sprout Invoices payment entry for the invoice when the order is set to complete.
	 *
	 * @param  int    $order_id The order id.
	 * @param  string $from_status The status the order is changing from.
	 * @param  string $to_status The status the order is changing to.
	 * @return void
	 */
	public static function maybe_create_si_payment( $order_id, $from_status = '', $to_status = '' ) {

		/**
		 * Check if a Woocommerce session exists. If maybe_create_si_payment is being called after to WooCommerce checkout
		 * we need to update the post_meta for the order to include information from Sprout Invoices invoice.
		 * The maybe_create_si_payment() function will run anytime an order status is changed. This includes the status changing
		 * outside of the WooCommerce checkout instance i.e. wp-admin order menu and Sprout invoice creation.
		 * Check if the order meta has the invoice id. If not, add it from the checkout session data.
		 */
		if ( isset( WC()->session ) && ! get_post_meta( $order_id, '_si_woo_invoice_id', true ) ) {
			update_post_meta( $order_id, '_si_woo_invoice_id', WC()->session->get( 'invoice_id' ) );
		}

		/**
		 * Check if a Woocommerce session exists. If maybe_create_si_payment() is being called after to WooCommerce checkout
		 * we need to update the post_meta for the order to include information from Sprout Invoices invoice.
		 * The maybe_create_si_payment() function will run anytime an order status is changed. This includes the status changing
		 * outside of the WooCommerce checkout instance i.e. wp-admin order menu and Sprout invoice creation.
		 * Check if the order meta has a deposit. If not, add it from the checkout session data.
		 */
		if ( isset( WC()->session ) && ! get_post_meta( $order_id, '_si_woo_deposit', true ) ) {
			update_post_meta( $order_id, '_si_woo_deposit', WC()->session->get( 'si_deposit' ) );
		}

		$order = wc_get_order( $order_id );

		/**
		 * Delete Generated Products from the Sprout Invoices.
		 * Generated Products are created when we redirect to Woocommerce. We are deleting
		 * them because we don't want to show them in the product catelog since they will
		 * always be unique.
		 */
		self::remove_line_item_products( $order );

		// If the order is not set to complete, return.
		if ( 'on-hold' === $to_status || 'failed' === $to_status ) {
			return;
		}

		$invoice_id = get_post_meta( $order_id, '_si_woo_invoice_id', true );
		$invoice    = SI_Invoice::get_instance( $invoice_id );

		// total includes taxes.
		$total = 0;
		if ( ! is_a( $invoice, 'SI_Invoice' ) ) {
			return;
		}

		if ( WC()->cart ) {
			$discount_excl_tax_total   = WC()->cart->get_cart_discount_total(); // total discount on woo invoice.
			$discount_tax_total        = WC()->cart->get_total_tax();  // tax on woo invoice.
			$subtotal                  = $order->get_subtotal();
			$discount_total            = $subtotal - $discount_excl_tax_total; // total amount that was discounted.
			$total                     = $order->get_total();
			$discount_total_percentage = ( 100 * ( $subtotal - $discount_total ) ) / $subtotal; // convert discount to percentage.

			// Set Discount unless the discount it's already included.
			if ( 0 < $discount_total_percentage ) {
				$invoice->set_discount( $discount_total_percentage );
			}
				// Set tax unless the tax it's already included.
				if ( ! $order->get_prices_include_tax() ) {
					$tax = (float) 0;
					if ( ! empty( $order->get_tax_totals() ) ) {
						$tax_total = (float) 0;
						foreach ( $order->get_tax_totals() as $code => $tax ) {
							$tax_total += wc_round_tax_total( $tax->amount );
						}
						$tax = ( $tax_total / (float) $order->get_subtotal() ) * 100;
					}
					if ( 0 < $tax ) {
						$invoice->set_tax( $tax );
					}
				}
			$invoice->set_total( $order->get_total() );
		} else {
			$total = $order->get_total();
		}

		// Check if the order is for a deposit
		$deposit = get_post_meta( $order_id, '_si_woo_deposit', true );

		if ( $invoice->get_status() !== 'complete' ) {
			// If the order is for a deposit, set the invoice to partial payment.
			if ( ! empty( $deposit ) ) {
				/**
				 * Check for partial payment. In woocommerec the order status can change from proccessing to complete.
				 * This is causing two payments to be created. We need to check if the invoice has the partial payment that
				 * is already associated with an order. We do not want to create another payment in this case.
				 */
				$partial_payments = $invoice->get_payments( array( 'type' => 'deposit' ) );
				if ( ! empty( $partial_payments ) ) {
					foreach ( $partial_payments as $partial_payment ) {
						$partial_payment_order_id = intval( SI_Payment::get_instance( $partial_payment )->get_data()['woo_order_id'] );
						if ( $partial_payment_order_id === $order_id ) {
							return;
						}
					}
				}
				// Create partial payment.
				$payment_status = SI_Payment::STATUS_PARTIAL;
				$payment_id     = self::create_payment_si_payment( $order, $invoice, $order_id, $payment_status );
				$add_deposit    = SI_Payment::get_instance( $payment_id )->set_type( 'deposit' );
			} else {
				// create new payment
				$payment_status = SI_Payment::STATUS_COMPLETE;
				$payment_id     = self::create_payment_si_payment( $order, $invoice, $order_id, $payment_status );
			}

			if ( ! $payment_id ) {
				return false;
			}
			$payment = SI_Payment::get_instance( $payment_id );
			do_action( 'payment_authorized', $payment );
			$payment->set_status( SI_Payment::STATUS_COMPLETE );
			do_action( 'payment_complete', $payment );
			return $payment;
		} else {
			return;
		}
	}

	/**
	 * Create payment for Sprout Invoices.
	 *
	 * This is a helper function for the maybe_create_si_payment function. If the payment for the Woocommerce order is successful.
	 * Create a Sprout Invoices payment entry for the invoice.
	 *
	 * @since 20.4
	 * @param  object $order The Woocommerce order object.
	 * @param  object $invoice The Sprout Invoices invoice object.
	 * @param  int    $order_id The Woocommerce order id.
	 * @param  string $payment_status The Sprout Invoices payment status.
	 *
	 * @return int The Sprout Invoices payment id.
	 */
	public static function create_payment_si_payment( $order, $invoice, $order_id, $payment_status ) {
		$payment_id = SI_Payment::new_payment(
			array(
				'payment_method' => self::PAYMENT_METHOD,
				'invoice_id'     => $invoice->get_id(),
				'amount'         => $order->get_total(),
				'transaction_id' => $order_id,
				'data'           => array(
					'amount'         => $order->get_total(),
					'woo_order_link' => '<a href="' . get_edit_post_link( $order_id ) . '">' . $order_id . '</a>',
					'woo_order_id'   => $order_id,
				),
			),
			$payment_status,
		);

		return $payment_id;
	}

	/**
	 * Remove line items products created by Sprout Invoices.
	 *
	 * This function removes the line items created by Sprout Invoices when the order is placed. This will prevent the line items from
	 * filling up the product table in Woocommerce. Since each line item is unique and not a product created in woocommerce it could
	 * become a problem. This also handles deletion of the product, if a customer doesnt complete the checkout process and the line items are still in the cart.
	 *
	 * @since 20.4
	 * @param  object $order The Woocommerce product object.
	 */
	public static function remove_line_item_products( $order = false ) {
		if ( ! $order ) {
			if ( ! WC()->cart->is_empty() ) {
				foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
					$product_id = $cart_item['product_id'];
					$product    = wc_get_product( $product_id );
					if ( $product->get_meta( 'deposit' ) === '1' || $product->get_meta( 'line_item' ) === '1' ) {
						$product->delete( true );
					}
				}
			}
		} else {
			foreach ( $order->get_items() as $item_id => $item ) {
				$product_id = $item->get_product_id();
				$product    = wc_get_product( $product_id );
				if ( ! empty( $product ) ) {
					if ( $product->get_meta( 'deposit' ) === '1' || $product->get_meta( 'line_item' ) === '1' ) {
						$product->delete( true );
					}
				}
			}
		}
	}
	/**
	 * Redirect to the invoice after payment.
	 *
	 * @param  int $order_id The order id.
	 * @return mixed
	 */
	public static function redirect_to_invoice_after_payment( $order_id = 0 ) {
		$order = wc_get_order( $order_id );

		$varient_id = 0;
		foreach ( $order->get_items() as $item ) {
			if ( isset( $item['variation_id'] ) ) {
				$varient_id = $item['variation_id'];
			}
		}

		if ( ! $varient_id ) {
			return;
		}

		$invoice_id = get_post_meta( $varient_id, '_invoice_id', true );
		$invoice    = SI_Invoice::get_instance( $invoice_id );

		if ( ! is_a( $invoice, 'SI_Invoice' ) ) {
			return;
		}

		$confirmation_url = esc_url_raw(
			add_query_arg(
				array(
					'invoice_payment'   => self::PAYMENT_SLUG,
					'si_payment_action' => 'confirmation',
				),
				get_permalink( $invoice_id )
			)
		);
		if ( $invoice->get_balance() < 0.01 ) {
			self::set_message( __( 'Payment Received & Invoice Paid!', 'sprout-invoices' ), self::MESSAGE_STATUS_INFO );
		} elseif ( $invoice->get_payments_total( false ) < 0.01 ) {
			self::set_message(
				sprintf(
					// translators: %s is the current balance.
					esc_html__( 'Payment Pending. Current Balance: %s', 'sprout-invoices' ),
					sa_get_formatted_money( $invoice->get_balance() )
				),
				self::MESSAGE_STATUS_INFO
			);
		} else {
			self::set_message(
				sprintf(
					// translators: %s is the current balance.
					esc_html__( 'Partial Payment Received. Current Balance: %s', 'sprout-invoices' ),
					sa_get_formatted_money( $invoice->get_balance() )
				),
				self::MESSAGE_STATUS_INFO
			);
		}

		if ( ! apply_filters( 'si_woopay_redirect_back_to_invoice', true ) ) {
			return;
		}

		if ( ! headers_sent() ) {
			wp_safe_redirect( $confirmation_url );
			exit();
		} else {
			echo "<script type='text/javascript'>window.location = '" . esc_url( $confirmation_url ) . "'</script>";
		}
	}
}
SI_Woo_Payment_Processor::register();
