<?php

/**
 * Woo_Products Controller
 *
 * @package Sprout_Invoice
 * @subpackage Woo_Products
 */
class Woo_Product_Line_Items extends SI_Controller {
	/**
	 * Class initialization
	 *
	 * @since 10.3.0
	 * @since 20.2.0 Added new get woo product ajax method and removed old.
	 *
	 * @return void
	 */
	public static function init() {

		if ( is_admin() ) {
			// Invoicing
			add_action( 'si_post_add_line_item', array( __CLASS__, 'add_woocommerce_products' ) );

			// Enqueue
			add_action( 'admin_enqueue_scripts', array( __CLASS__, 'register_resources' ) );
			add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_enqueue' ), 20 );
		}
		// Add hidden Woocommerce product ID field to line items.
		add_filter( 'si_line_item_columns', array( __CLASS__, 'add_woo_productid_column' ), 10, 3 );

		// ajax actions
		add_action( 'wp_ajax_get_si_woo_product', array( __CLASS__, 'get_si_woo_product' ), 10, 0 );

	}

	/**
	 * Register scripts and styles.
	 *
	 * @since 10.3.0
	 * @since 20.2.0 Enqueue WooCommerce Product modal script.
	 *
	 * @return void
	 */
	public static function register_resources() {
		// WooCommerce Product modal CSS
		wp_enqueue_style( 'si_woocommerce_products', SA_ADDON_WOOCOMMERCE_URL . '/resources/admin/css/si_woo_product_modal.css', array(), self::SI_VERSION );
	}

	/**
	 * Enqueue admin scripts and styles.
	 *
	 * @since 10.3.0
	 *
	 * @return void
	 */
	public static function admin_enqueue() {

		// doc admin templates
		$screen           = get_current_screen();
		$screen_post_type = str_replace( 'edit-', '', $screen->id );
		if ( in_array( $screen_post_type, array( SI_Estimate::POST_TYPE, SI_Invoice::POST_TYPE ) ) ) {
			add_thickbox();
		}
	}

	/**
	 * Add WooCommerce products to the line items editor.
	 *
	 * @since 20.2.0
	 *
	 * @return void
	 */
	public static function add_woocommerce_products() {
		$products      = self::get_woo_products();
		$categories    = get_terms(
			'product_cat',
			array(
				'hide_empty' => true,
			)
		);
		$products_info = array();
		foreach ( $categories as $category ) {
			$products_info += self::si_get_woo_products_by_category( $products, $category->slug );
		}

		// Line Items Vue.
		wp_enqueue_script( 'sprout-invoices-woocommerce-products', SA_ADDON_WOOCOMMERCE_URL . '/resources/admin/js/si_woo_product_modal.js', array( 'sprout-invoices-vue', 'jquery', 'si_admin' ), self::SI_VERSION, true );

		// Sending data to our plugin settings JS file.
		wp_localize_script(
			'sprout-invoices-woocommerce-products',
			'SI_WOO_PRODUCTS',
			array(
				'nonce'    => wp_create_nonce( 'wp_rest' ),
				'restURL'  => esc_url_raw( rest_url() ),
				'products' => $products_info,
			)
		);

		// load WooCommerce Product modal.
		self::load_addon_view(
			'admin/meta-boxes/invoices/add-product.php',
			array(
				'products'   => $products_info,
				'categories' => $categories,
			),
			false
		);
	}

	/**
	 * Loop through WooCommerce products and return them in an array.
	 *
	 * @param array $products WooCommerce products.
	 * @param array $category WooCommerce category.
	 * @todo Refator this to use multiple methods class.
	 * @since 20.2.0
	 * @return array $products_info WooCommerce products info.
	 */
	public static function si_get_woo_products_by_category( $products, $category ) {
		$products_info = array();

		// Loop through products in each category. If the category is empty, skip it.
		if ( ! is_null( $products[ $category ] ) ) {
			foreach ( array_keys( $products[ $category ] ) as $product_id ) {

				$product = wc_get_product( $product_id );

				if ( empty( $product ) ) {
					continue;
				}
				if ( $product instanceof WC_Product_Variable ) {
					$var_atts_keys = $product->get_attributes();
					$var_atts      = array_keys( $var_atts_keys );
					$var_array     = array();

					foreach ( $var_atts as $key => $value ) {
						$var_array += array(
							'attribute_' . $value => array(
								'name'    => ucfirst( str_replace( '-', ' ', $value ) ),
								'options' => explode( ' | ', $product->get_attribute( $value ) ),
							),
						);
					}

					$products_info[ $product_id ]['variations'] = $product->get_available_variations();
					foreach ( $products_info[ $product_id ]['variations'] as $child_id => $child_info ) {
						$products_info[ $product_id ]['variations'][ $child_info['variation_id'] ] = $child_info;
						$products_info[ $product_id ]['variations']['variation_options']           = $var_array;
						unset( $products_info[ $product_id ]['variations'][ $child_id ] );
					}
				} else {
					$products_info[ $product_id ]['price'] = $product->get_price();
				}
				$products_info[ $product_id ]['name']       = $product->get_name();
				$products_info[ $product_id ]['type']       = $product->get_type();
				$products_info[ $product_id ]['desc']       = $product->get_description();
				$products_info[ $product_id ]['short_desc'] = $product->get_short_description();

			}
		}
		return $products_info;
	}

	/**
	 * Ajax callback to get a product information from WooCommerce.
	 *
	 * @since 20.2.0
	 *
	 * @return void
	 */
	public static function get_si_woo_product() {
		if ( ! current_user_can( 'publish_sprout_invoices' ) ) {
			wp_send_json_error( array( 'message' => __( 'User cannot create invoices!', 'sprout-invoices' ) ) );
		}

		if ( ! wp_verify_nonce( $_POST['nonce'], 'wp_rest' ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid nonce!', 'sprout-invoices' ) ) );
		}

		if ( ! isset( $_POST['productId'] ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid Product ID!', 'sprout-invoices' ) ) );
		}

		$product_id = (int) $_POST['productId'];
		$product    = wc_get_product( $product_id );
		if ( ! $product ) {
			wp_send_json_error( array( 'message' => __( 'Invalid Product!', 'sprout-invoices' ) ) );
		}

		if ( 'true' === $_POST['productType'] ) {
			$variation = new WC_Product_Variation( ( new \WC_Product_Data_Store_CPT() )->find_matching_product_variation( $product, $_POST['productVariation'] ) );

			$selected_variations = array();
			foreach ( $_POST['productVariation'] as $key => $value ) {
				$selected_variations[ $key ] = $value;
			};

			$product_info = array(
				'type'                    => 'product',
				'desc'                    => sprintf(
					'<b>%1$s:</b> %2$s',
					$variation->get_name(),
					implode( ' - ', $selected_variations )
				),
				'title'                   => $variation->get_name(),
				'rate'                    => wc_format_decimal( $variation->get_price(), 2 ),
				'price'                   => wc_format_decimal( $variation->get_price(), 2 ),
				'qty'                     => $_POST['productQty'],
				'sku'                     => $product->get_sku(),
				'wc_product'              => $product->get_id(),
				'wc_variation'            => $variation->get_id(),
				'wc_variation_attributes' => wp_json_encode( $selected_variations ),
			);
		} else {
			$product_info = array(
				'type'       => 'product',
				'desc'       => sprintf(
					'<b>%1$s</b>',
					$product->get_name()
				),
				'title'      => $product->get_name(),
				'rate'       => wc_format_decimal( $product->get_price(), 2 ),
				'price'      => wc_format_decimal( $product->get_price(), 2 ),
				'qty'        => $_POST['productQty'],
				'sku'        => $product->get_sku(),
				'wc_product' => $product->get_id(),
			);
		}

		ob_start();
		SI_Controller::load_view(
			'admin/sections/line-item-options',
			array(
				'columns'      => SI_Line_Items::line_item_columns( $product_info['type'] ),
				'item_data'    => $product_info,
				'has_children' => false,
				'items'        => array(),
				'position'     => 0,
				'children'     => array(),
			),
			false
		);
		$option = ob_get_clean();

		$view = sprintf( '<li id="line_item_loaded_%1$s" class="item product line_item_type_%1$s" data-id="0">%2$s</li>', $product_info['type'], $option );

		wp_send_json_success(
			array(
				'option' => $view,
				'type'   => $product_info['type'],
			)
		);
	}

	/**
	 * Add Woocommerce product ID to line item.
	 *
	 * @param array  $columns Line item columns.
	 * @param string $type Line item type.
	 * @param array  $item_data Line item data.
	 *
	 * @return array $columns Line item columns.
	 */
	public static function add_woo_productid_column( $columns = array(), $type = '', $item_data = array() ) {
		$columns['wc_product']              = array(
			'label'          => esc_html__( 'Woocommerce Product ID', 'sprout-invoices' ),
			'type'           => 'hidden',
			'calc'           => false,
			'hide_if_parent' => true,
			'weight'         => 21,
		);
		$columns['wc_variation']            = array(
			'label'          => esc_html__( 'Woocommerce Variation ID', 'sprout-invoices' ),
			'type'           => 'hidden',
			'calc'           => false,
			'hide_if_parent' => true,
			'weight'         => 21,
		);
		$columns['wc_variation_attributes'] = array(
			'label'          => esc_html__( 'Woocommerce Variation Attributes', 'sprout-invoices' ),
			'type'           => 'hidden',
			'calc'           => false,
			'hide_if_parent' => true,
			'weight'         => 21,
		);
		return $columns;
	}
	/**
	 * Get post data.
	 *
	 * @since  3.0.0
	 * @return $post_data object Post data.
	 */
	public function get_post_data() {
		wc_deprecated_function( 'WC_Product::get_post_data', '3.0', 'get_post' );

		// In order to keep backwards compatibility it's required to use the parent data for variations.
		if ( $this->is_type( 'variation' ) ) {
			$post_data = get_post( $this->get_parent_id() );
		} else {
			$post_data = get_post( $this->get_id() );
		}

		return $post_data;
	}

	/**
	 * Get Woo products and create array of posts.
	 *
	 * @since 10.3.0
	 *
	 * @return array $products woocommerce products array.
	 */
	public static function get_woo_products() {
		$cache_key       = '_si_cached_woo_products';
		$cached_products = get_transient( $cache_key );
		if ( $cached_products ) {
			if ( ! empty( $cached_products ) ) {
				return $cached_products;
			}
		}

		$products = array(); // multi-dimensional array of products and tasks
		$args     = array(
			'post_type'      => 'product',
			'posts_per_page' => -1,
			'fields'         => 'ids',
			'orderby'        => 'modified',
		);

		$product_ids = get_posts( $args );

		foreach ( $product_ids as $product_id ) {
			$product    = new WC_Product( $product_id );
			$categories = wp_get_post_terms( $product_id, 'product_cat' );
			if ( ! empty( $categories ) ) {
				// create a product within each category assigned.
				foreach ( $categories as $category ) {
					$products[ $category->slug ][ $product_id ] = array(
						'id'         => $product_id,
						'qty'        => 1,
						'rate'       => wc_format_decimal( $product->get_price(), 2 ),
						'percentage' => 0,
						'sku'        => $product->get_sku(),
						'title'      => $product->get_title(),
						'content'    => apply_filters( 'woocommerce_short_description', get_post( $product->get_id() )->post_excerpt ),
					);
				}
			} else {
				$products['none'][ $product_id ] = array(
					'id'         => $product_id,
					'qty'        => 1,
					'rate'       => wc_format_decimal( $product->get_price(), 2 ),
					'percentage' => 0,
					'sku'        => $product->get_sku(),
					'title'      => $product->get_title(),
					'content'    => apply_filters( 'si_woocommerce_short_description', get_post( $product->get_id() )->post_excerpt, $product ),
				);
			}
		}
		set_transient( $cache_key, $products, HOUR_IN_SECONDS * 2 );
		return apply_filters( 'si_woocommerce_products', $products );
	}

	/**
	 * Load addon view from views folder.
	 *
	 * @since 10.3.0
	 *
	 * @param  string $view relative path to view file.
	 * @param  array  $args array of arguments to pass to view.
	 * @param  bool   $allow_theme_override allow theme override.
	 * @return string $view view html in string format.
	 */
	public static function load_addon_view( $view, $args, $allow_theme_override = true ) {
		add_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		$view = self::load_view( $view, $args, $allow_theme_override );
		remove_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		return $view;
	}

	/**
	 * Addons view path.
	 *
	 * @since 10.3.0
	 *
	 * @return string path to addons view folder.
	 */
	public static function addons_view_path() {
		return SA_ADDON_WOOCOMMERCE_PATH . '/views/';
	}
}
