/**
 * Woocommerce Product Dropdown.
 *
 * This file contains the JS for the Woocommerce and Sprout Invoice integration dropdown in the edit invoice page.
 * The dropdown contains products created in Woocommerce to be added as line items in Sprout Invoices.
 *
 * @author Sprout Invoices.
 * @since  20.2.0
 */

/**
 * Format the dropdown results from select2.
 *
 * @access private
 * @since 20.3.0
 * @param {string} item
 * @return {string} returnString
 */
function formatResult( item ) {

	const selectionText = item.text.split( '.' ),
		returnString = jQuery( '<span>' + selectionText[ 0 ] + '</br>' + selectionText[ 1 ] + '</span>' );

	return returnString;

}

/**
 * Format the dropdown selection from select2.
 *
 * @since 20.3.0
 * @access private
 * @param {string} item
 * @return {string} returnString
 */
function formatSelection( item ) {

	const selectionText = item.text.split( '.' ),
		returnString = jQuery( '<span>' + selectionText[ 0 ] + '</span>' );

	return returnString;

}

/**
 * Create select2 component for modal dropdown.
 *
 * @see https://v2.vuejs.org/v2/guide/components-registration.html
 * @access private
 */
const Select2 = ( 'select2', {
	props: [ 'options', 'value' ],
	template: '<select><slot></slot></select>',
	mounted() {

		const vm = this;
		jQuery( this.$el )

		// init select2.
			.select2( {
				data: this.products,
				templateResult: formatResult,
				templateSelection: formatSelection
			} )
			.val( this.value )
			.trigger( 'change' )

		// emit event on change.

			.on( 'change', function() {

				vm.$emit( 'input', this.value );

			} );

	},
	watch: {
		value( value ) {

			// update value.
			jQuery( this.$el )
				.val( value )
				.trigger( 'change' );
			this.$emit( 'change', value );

		},
		options( options ) {

			// update options.
			jQuery( this.$el )
				.empty()
				.select2( { data: options } );

		}
	},
	destroyed() {

		jQuery( this.$el )
			.off()
			.select2( 'destroy' );

	}
} );

/**
 * Create the vue instance.
 *
 * @see https://v2.vuejs.org/v2/guide/instance.html
 */
new window.Vue( {

	/**
	 * DOM selector for our app's main wrapper element.
	 *
	 * @since 20.2.0
	 * @access protected
	 * @type {string}
	 */
	el: '#si_woo_product_vue',

	components: {
		select2: Select2
	},

	/**
	 * Data that will be proxied by Vue.js to provide reactivity to our template.
	 *
	 * @since 20.2.0
	 * @access protected
	 */
	data() {

		return this.initialState();

	},

	/**
	 * Methods that can be invoked from within our template.
	 *
	 * @since 20.2.0
	 * @access private
	 */
	methods: {

		/**
		 * Set the intial data state.
		 *
		 * @since 20.3.0
		 */
		initialState() {

			return {
				showModal: false,
				selectedProduct: false,
				products: window.SI_WOO_PRODUCTS.products,
				variable: false,
				variableInfo: false,
				qty: '',
				variableSelection: {}
			};

		},

		/**
		 * Set show modal to true.
		 *
		 * @since 20.3.0
		 */
		showWooProductModal() {

			this.showModal = true;

		},

		/**
		 * Reset the Vue data when modal is closed.
		 *
		 * @since 20.3.0
		 */
		closeWooProductModal() {

			Object.assign( this.$data, this.initialState() );

		},

		/**
		 * Add the selected product to the invoice.
		 *
		 * @since 20.2.0
		 */
		addProduct() {

			const typeList = jQuery( 'ol#line_item_list' );

			jQuery.ajax( {

				url: window.ajaxurl,
				method: 'POST',
				data: { productVariation: this.variableSelection, productQty: this.qty, productType: this.variable, productId: this.selectedProduct, action: 'get_si_woo_product', nonce: window.SI_WOO_PRODUCTS.nonce },

				success: ( response ) => {

					this.showModal = false;
					const row = jQuery( response.data.option );

					// append the row to the list.
					typeList.append( row );

					// Add the redactor
					if ( window.si_js_object.redactor ) {

						row.find( '.column_desc [name="line_item_desc[]"]' ).redactor();

					}
					this.modifyInputKey();

					// calculate totals
					this.calculateEachLineItemTotal();
					this.calculateLineItemTotals();

					// Rest the vue data.
					Object.assign( this.$data, this.initialState() );

				},

				error: ( response ) => {

					typeList.append( '<span class="inline_message inline_error_message">' + response.data.message + '</span>' );

				}
			} );

		},

		/**
		 * Load Product Variations if the product is variable.
		 *
		 * @since 20.2.0
		 *
		 * @param {Integer} val product id.
		 */
		loadProductVariations( val ) {

			const productType = this.products[ val ].type;
			if ( 'variable' === productType ) {

				this.variable = true;
				this.variableInfo = this.products[ val ].variations.variation_options;

			} else {

				this.variable = false;

			}

		},

		/**
		 * Store line item key.
		 *
		 * @since 20.2.0
		 */
		modifyInputKey() {

			let level1,
				$index;
			jQuery( 'ol.items_list' ).each( function( ol ) {

				ol = jQuery( ol );
				level1 = ol.closest( 'li' ).index() + 1;

				ol.children( 'li' ).each( function( li ) {

					li = jQuery( li );
					$index = ( level1 === 0 ) ? li.index() + 1 : level1 + '.' + ( li.index() + 1 );
					li.find( '.line_item_index' ).val( $index );

				} );

			} );

		},

		/**
		 * Calculate the total for each line item if it has children items.
		 *
		 * @since 20.2.0
		 */
		calculateEachLineItemTotal() {

			jQuery( 'ol.items_list .item' ).each( function( i, li ) {

				// If no children
				if ( jQuery( li ).children( 'ol' ).length === 0 ) {

					window.si.lineItems.calculateLineItemTotals( li );
					window.si.lineItems.calculateParentTotals();

				}

			} );

		},

		/**
		 * Calculate the total each line item.
		 *
		 * @since 20.2.0
		 *
		 * @param {string} li list item html.
		 */
		calculateLineItemTotals( li ) {

			let $rateTotal,
				$qtyTotal,
				$taxTotal;

			// Clear out the totals if there's no more children
			jQuery( li ).find( '.column_rate span', '.column_qty span', '.column_tax span', '.column_total span' ).html( '' );

			// do the totals
			$rateTotal = jQuery( li ).find( '.column_rate input' ).val();
			if ( $rateTotal === undefined ) {

				$rateTotal = 0;

			}
			$qtyTotal = jQuery( li ).find( '.column_qty input' ).val();
			if ( $qtyTotal === undefined ) {

				$qtyTotal = 1;

			}
			$taxTotal = jQuery( li ).find( '.column_tax input' ).val();
			if ( $taxTotal === undefined ) {

				$taxTotal = 0;

			}
			const total = ( $rateTotal * $qtyTotal ) * ( ( 100 - $taxTotal ) / 100 );

			jQuery( li ).find( '.column_total span' ).html( parseFloat( total ).toFixed( 2 ) );
			jQuery( li ).find( '.column_total input' ).val( parseFloat( total ).toFixed( 2 ) );

			jQuery( 'ol.items_list' ).trigger( 'siCalculateLineItemTotals', [ li, total ] );

		}
	}
} );
