<?php
require_once 'inc/sdk/autoload.php';
use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;

/**
 * Authorize.net onsite credit card payment processor.
 *
 * @package SI
 * @subpackage Payment SI_Credit_Card_Processors
 */
class SA_AuthorizeNet extends SI_Credit_Card_Processors {

	const API_ENDPOINT_SANDBOX = 'https://test.authorize.net/gateway/transact.dll';
	const API_ENDPOINT_LIVE    = 'https://secure.authorize.net/gateway/transact.dll';
	const MODE_TEST            = 'sandbox';
	const MODE_LIVE            = 'live';
	const API_USERNAME_OPTION  = 'si_authorize_net_username';
	const API_PASSWORD_OPTION  = 'si_authorize_net_password';
	const API_MODE_OPTION      = 'si_authorize_net_mode';
	const PAYMENT_METHOD       = 'Credit (Authorize.Net)';
	const PAYMENT_SLUG         = 'authnet';

	protected static $instance;
	private static $api_mode     = self::MODE_TEST;
	private static $api_username = '';
	private static $api_password = '';

	/**
	 * Get the class instance.
	 *
	 * @return SA_AuthorizeNet
	 */
	public static function get_instance() {
		if ( ! ( isset( self::$instance ) && is_a( self::$instance, __CLASS__ ) ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Get type authorize.net api url
	 *
	 * Returns the api url based on the mode set (sandbox or live).
	 *
	 * @return string api url.
	 */
	private function get_api_url() {
		if ( self::$api_mode == self::MODE_LIVE ) {
			return self::API_ENDPOINT_LIVE;
		} else {
			return self::API_ENDPOINT_SANDBOX;
		}
	}

	/**
	 * Get the payment method.
	 *
	 * @return string
	 */
	public function get_payment_method() {
		return self::PAYMENT_METHOD;
	}

	/**
	 * Get the payment slug.
	 *
	 * @return string
	 */
	public function get_slug() {
		return self::PAYMENT_SLUG;
	}

	/**.
	 * Register the payment processor.
	 *
	 * @return void
	 */
	public static function register() {
		self::add_payment_processor( __CLASS__, __( 'Authorize.net', 'sprout-invoices' ) );
	}

	/**
	 * Set the public name of the payment method.
	 *
	 * @return string
	 */
	public static function public_name() {
		return __( 'Credit Card', 'sprout-invoices' );
	}

	/**
	 * Set checkout options.
	 *
	 * @return array $options checkout options.
	 */
	public static function checkout_options() {
		$option = array(
			'icons'          => array(
				SI_URL . '/resources/front-end/img/visa.png',
				SI_URL . '/resources/front-end/img/mastercard.png',
				SI_URL . '/resources/front-end/img/amex.png',
				SI_URL . '/resources/front-end/img/discover.png',
			),
			'label'          => __( 'Credit Card', 'sprout-invoices' ),
			'accepted_cards' => array(
				'visa',
				'mastercard',
				'amex',
				'diners',
				'jcb',
			),
		);
		return $option;
	}

	/**
	 * Class constructor.
	 */
	protected function __construct() {
		parent::__construct();
		self::$api_username = get_option( self::API_USERNAME_OPTION, '' );
		self::$api_password = get_option( self::API_PASSWORD_OPTION, '' );
		self::$api_mode     = get_option( self::API_MODE_OPTION, self::MODE_TEST );

		// Add Authentication
		add_filter( 'si_api_authentication', array( $this, 'authenticate_anet' ) );

		// Remove pages
		add_filter( 'si_checkout_pages', array( $this, 'remove_checkout_pages' ) );
	}

	/**
	 * The review page is unnecessary
	 *
	 * @param array $pages sprout invoice pages.
	 * @return array $pages sprout invoice pages.
	 */
	public function remove_checkout_pages( $pages ) {
		unset( $pages[ SI_Checkouts::REVIEW_PAGE ] );
		return $pages;
	}

	/**
	 * Hooked on init add the settings page and options.
	 *
	 * @param  array $settings the settings array.
	 * @return array $settings the settings array.
	 */
	public static function register_settings( $settings = array() ) {

		// Settings
		$settings['payments'] = array(
			'si_authorizenet_settings' => array(
				'title'    => __( 'Authorize.net', 'sprout-invoices' ),
				'weight'   => 200,
				'settings' => array(
					self::API_MODE_OPTION     => array(
						'label'  => __( 'Mode', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'select',
							'options' => array(
								self::MODE_LIVE => __( 'Live', 'sprout-invoices' ),
								self::MODE_TEST => __( 'Sandbox', 'sprout-invoices' ),
							),
							'default' => self::$api_mode,
						),
					),
					self::API_USERNAME_OPTION => array(
						'label'   => __( 'API Login ID', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => get_option( self::API_USERNAME_OPTION, '' ),
						),
					),
					self::API_PASSWORD_OPTION => array(
						'label'  => __( 'Transaction Key', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => get_option( self::API_PASSWORD_OPTION, '' ),
						),
					),
				),
			),
		);
		return $settings;
	}

	/**
	 * Get authentication credentials for the Authorize.net API
	 *
	 * @return array
	 */
	public static function get_anet_credentials() {
		self::$api_mode      = get_option( self::API_MODE_OPTION, self::MODE_TEST );
		self::$api_username  = get_option( self::API_USERNAME_OPTION, '' );
		self::$api_password  = get_option( self::API_PASSWORD_OPTION, '' );
		$si_anet_credentials = array();
		if ( ! empty( self::$api_username ) || ! empty( self::$api_password ) ) {
			$merchant_authentication = new AnetAPI\MerchantAuthenticationType();
			$merchant_authentication->setName( self::$api_username );
			$merchant_authentication->setTransactionKey( self::$api_password );
			$ref_id = 'ref' . time();

			$si_anet_credentials = array(
				'merchant_authentication' => $merchant_authentication,
				'ref_id' => $ref_id,
			);
		}
		return $si_anet_credentials;
	}

	/**
	 * Authenticate API credentials. This is used in the settings menu to verify credentials after they are saved.
	 * If the credentials are valid, the options are saved. If not, a message will display saving invalid credentials.
	 *
	 * @return bool
	 */
	public static function authenticate_anet() {
		$si_anet_credentials = self::get_anet_credentials();
		if ( ! empty( $si_anet_credentials ) ) {

			// Get all existing customer profile ID's for auth test.
			$request = new AnetAPI\GetCustomerProfileIdsRequest();
			$request->setMerchantAuthentication( $si_anet_credentials['merchant_authentication'] );
			$controller = new AnetController\GetCustomerProfileIdsController( $request );
			if ( self::MODE_TEST === self::$api_mode ) {
				$response = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::SANDBOX );
			} else {
				$response = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::PRODUCTION );
			}
			if ( $response->getMessages()->getResultCode() === 'Error' ) {
				$authentication = $response->getMessages()->getMessage()[0]->getText();
				return $authentication;
			}
		}
	}

	/**
	 * Process a payment
	 *
	 * @param SI_Checkouts $checkout The checkout object.
	 * @param SI_Invoice   $invoice The invoice object.
	 * @return SI_Payment|bool FALSE if the payment failed, otherwise a Payment object
	 */
	public function process_payment( SI_Checkouts $checkout, SI_Invoice $invoice ) {

		$post_data = $this->si_anet_transaction( $checkout, $invoice );

		// If transaction data is null return false. This means the transaction failed at Authorize.net and we are showing them the error message already.
		if ( null === $post_data ) {
			return false;
		}
		// If the transaction failed but has a transaction id return false and set the error message for the customer.
		if ( null !== $post_data->getErrors() ) {
			foreach ( $post_data->getErrors() as $error ) {
				self::set_error_messages( $error->getErrorText() );
			}
			return false;
		};

		// Create the payment record in sprout invoices.
		$payment_id = SI_Payment::new_payment(
			array(
				'payment_method' => $this->get_payment_method(),
				'invoice'        => $invoice->get_id(),
				'amount'         => ( si_has_invoice_deposit( $invoice->get_id() ) ) ? $invoice->get_deposit() : $invoice->get_balance(),
				'transaction_id' => $post_data->getTransId(),
				'data'           => array(
					'live'         => ( self::MODE_LIVE === self::$api_mode ),
					'api_response' => self::get_transaction_details( $post_data->getTransId() ),
				),
			),
			SI_Payment::STATUS_AUTHORIZED
		);
		if ( ! $payment_id ) {
			return false;
		}

		// Go through the routine and do the authorized actions and then complete.
		$payment = SI_Payment::get_instance( $payment_id );
		do_action( 'payment_authorized', $payment );
		$payment->set_status( SI_Payment::STATUS_COMPLETE );
		do_action( 'payment_complete', $payment );

		return $payment;
	}

	/**
	 * Create AuthorizeNet payment data. This will create the payment data for the transaction.
	 *
	 * @return object $payment  The payment data object.
	 */
	public function si_anet_payment() {

		// Create the payment data for a credit card
		$credit_card = new AnetAPI\CreditCardType();
		$credit_card->setCardNumber( $this->cc_cache['cc_number'] );
		$credit_card->setExpirationDate( $this->cc_cache['cc_expiration_year'] . '-' . $this->cc_cache['cc_expiration_month'] );
		$credit_card->setCardCode( $this->cc_cache['cc_cvv'] );

		// Add the payment data to a paymentType object
		$payment = new AnetAPI\PaymentType();
		$payment->setCreditCard( $credit_card );

		return $payment;
	}

	/**
	 * Create Customer profile data. This will create the customer profile data for the transaction.
	 *
	 * @param  object $invoice  The invoice object.
	 */
	public static function si_anet_customer_profile( $invoice ) {
		$user = si_who_is_paying( $invoice );

		// User email or none.
		$user_email = ( $user ) ? $user->user_email : '';

		// Set the customer's identifying information.
		$customer_data = new AnetAPI\CustomerDataType();
		$customer_data->setType( 'individual' );
		$customer_data->setId( strval( mt_rand() ) );
		$customer_data->setEmail( $user_email );
	}

	/**
	 * Create AuthorizeNet customer address object.
	 *
	 * @param  object $checkout The checkout object.
	 * @return object $transactionRequestType  The transaction data object.
	 */
	public static function get_si_anet_billing_info( $checkout ) {

		// Set the customer's Bill To address.
		$customer_billing = new AnetAPI\CustomerAddressType();
		$customer_billing->setFirstName( $checkout->cache['billing']['first_name'] );
		$customer_billing->setLastName( $checkout->cache['billing']['last_name'] );
		$customer_billing->setAddress( $checkout->cache['billing']['street'] );
		$customer_billing->setCity( $checkout->cache['billing']['city'] );
		$customer_billing->setState( $checkout->cache['billing']['zone'] );
		$customer_billing->setZip( $checkout->cache['billing']['postal_code'] );
		$customer_billing->setCountry( $checkout->cache['billing']['country'] );

		return $customer_billing;
	}

	/**
	 * Create AuthorizeNet transaction data. This will create the transaction data for the transaction.
	 *
	 * @param  SI_Checkouts $checkout The checkout object.
	 * @param  SI_Invoice   $invoice  The invoice object.
	 * @return array        $tresponse  The transaction data.
	 */
	private function si_anet_transaction( SI_Checkouts $checkout, SI_Invoice $invoice ) {

		// Authorize.net credentials.
		$si_anet_credentials = $this->get_anet_credentials();

		// Get customer billing information.
		$si_anet_billing_info = $this->get_si_anet_billing_info( $checkout );

		// Create payment.
		$payment = $this->si_anet_payment();

		// Get payment amount.
		$payment_amount = ( si_has_invoice_deposit( $invoice->get_id() ) ) ? $invoice->get_deposit() : $invoice->get_balance();

		// Create the transaction data.
		$transaction_request_type = new AnetAPI\TransactionRequestType();
		$transaction_request_type->setTransactionType( 'authCaptureTransaction' );
		$transaction_request_type->setAmount( $payment_amount );
		$transaction_request_type->setPayment( $payment );
		$transaction_request_type->setBillTo( $si_anet_billing_info );

		// Create the request with the credentials and transaction data.
		$request = new AnetAPI\CreateTransactionRequest();
		$request->setMerchantAuthentication( $si_anet_credentials['merchant_authentication'] );
		$request->setRefId( $si_anet_credentials['ref_id'] );
		$request->setTransactionRequest( $transaction_request_type );

		// Create the controller and get the response.
		$controller = new AnetController\CreateTransactionController( $request );
		if ( self::MODE_TEST === self::$api_mode ) {
			$response = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::SANDBOX );
		} else {
			$response = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::PRODUCTION );
		}

		if ( null !== $response ) {

			// Check to see if the API request was successfully received and acted upon
			if ( $response->getMessages()->getResultCode() === 'Ok' ) {
				$tresponse = $response->getTransactionResponse();
				return $tresponse;
			} else {
				// If an error occured, get the error code and display it.
				$error_messages = $response->getMessages()->getMessage();
				$error_code     = $error_messages[0]->getCode();
				$error_text     = $error_messages[0]->getText();
				$error          = $error_code . ' ' . $error_text;

				if ( null !== $response->getTransactionResponse()->getErrors() ) {
					foreach ( $response->getTransactionResponse()->getErrors() as $error ) {
						self::set_error_messages( $error->getErrorText() );
					}
				}

				// Set the error message.
				self::set_error_messages( $error );

			}
		}

	}

	/**
	 * Get AuthorizeNet transaction details.
	 *
	 * @param  int $transaction_id The transaction id.
	 * @return object $transaction_details The transaction details.
	 */
	private function get_transaction_details( $transaction_id ) {
		$request = new AnetAPI\GetTransactionDetailsRequest();
		$request->setMerchantAuthentication( $this->get_anet_credentials()['merchant_authentication'] );
		$request->setTransId( $transaction_id );

		$controller = new AnetController\GetTransactionDetailsController( $request );

		if ( self::MODE_TEST === self::$api_mode ) {
			$response            = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::SANDBOX );
			$transaction_details = array(
				'Transaction Status' => $response->getTransaction()->getTransactionStatus(),
				'Transaction Amount' => $response->getTransaction()->getAuthAmount(),
				'Transaction ID'     => $response->getTransaction()->getTransId(),
			);
		} else {
			$response            = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::PRODUCTION );
			$transaction_details = array(
				'Transaction Status' => $response->getTransaction()->getTransactionStatus(),
				'Transaction Amount' => $response->getTransaction()->getAuthAmount(),
				'Transaction ID'     => $response->getTransaction()->getTransId(),
			);
		}
		return $transaction_details;
	}

	/**
	 * Get currency code from the invoice.
	 *
	 * @param  int $invoice_id The invoice id.
	 * @return string $currency_code The currency code for the invoice.
	 */
	private function get_currency_code( $invoice_id ) {
		return apply_filters( 'si_currency_code', self::$currency_code, $invoice_id, self::PAYMENT_METHOD );
	}

	/**
	 * Grabs error messages from a Authorize response and displays them to the user
	 *
	 * @param array $response The response from Authorize.
	 * @param bool  $display Display the error message.
	 * @return void
	 */
	private function set_error_messages( $response, $display = true ) {
		if ( $display ) {
			self::set_message( $response, self::MESSAGE_STATUS_ERROR );
		} else {
			do_action( 'si_error', __CLASS__ . '::' . __FUNCTION__ . ' - Auth.net Error Response', $response );
		}
	}
}
SA_AuthorizeNet::register();
