/**
 * PayPal Checkout JavaScript
 *
 * This file uses the PayPal JavaScript SDK to create a PayPal Checkout button and back-end calls to create and capture orders.
 *
 * @since 20.6.1
 * @package PayPalPayments
 *
 *
 */
window.paypal.Buttons( {
  style: {
    layout: 'vertical',
    color:  'blue',
    shape:  'rect',
    label:  'paypal',
    disableMaxWidth: true
  },
  /**
   * Create Order and Send information to the server.
   *
   * @since 20.6
   *
   * @returns {Object} orderData.data - The order ID from the server.
   */
  async createOrder() {
    try {
      const response = await fetch( si_js_object.rest_url + "sprout/v1/paypal-checkout/create-order", {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify( {
          invoice_id: si_js_object.invoice_id,
        } ),
      } );

      const orderData = await response.json();

      if ( orderData.data ) {
        return orderData.data;
      } else {
        const errorDetail = orderData?.details?.[0];
        const errorMessage = errorDetail
          ? `${ errorDetail.issue } ${ errorDetail.description } ( ${ orderData.debug_id } )`
          : JSON.stringify( orderData );

        throw new Error( errorMessage );
      }
    } catch ( error ) {
        console.error( error );
    }
  },
  /**
   * Once order is appproved send information to the server to capture the order and complete the transaction.
   *
   * @since 20.6
   *
   * @returns void actions.restart() - Reloads the page for the invoice.
   */
  async onApprove( data, actions ) {
    try {
      const response = await fetch( si_js_object.rest_url + 'sprout/v1/paypal-checkout/capture-order', {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify( {
          paypal_order_id: data.orderID,
        } ),
      } );

      const orderData = await response.json();

      /*
       * Three cases to handle:
       *  (1) Recoverable INSTRUMENT_DECLINED -> call actions.restart().
       *  (2) Other non-recoverable errors -> Show a failure message.
       *  (3) Successful transaction -> Show confirmation or thank you message.
       */
      const errorDetail = orderData.data.details?.[0];

      if ( "INSTRUMENT_DECLINED" === errorDetail?.issue ) {
        // (1) Recoverable INSTRUMENT_DECLINED -> call actions.restart()
        return actions.restart();
      } else if ( errorDetail ) {
        // (2) Other non-recoverable errors -> Show a failure message
        throw new Error( `${ errorDetail.description } ( ${ orderData.debug_id } )` );
      } else if ( ! orderData.data.purchase_units ) {
        throw new Error( JSON.stringify( orderData ) );
      } else {
        // (3) Successful transaction -> Show confirmation or thank you message
        location.reload();
      }
    } catch ( error ) {
      console.error( error );
    }
  },
  /**
   * Redirect to Invoice and show payment cancelled message.
   *
   * @since 20.6
   *
   * @returns void location.reload() - Reloads the page for the invoice.
   */
  async onCancel( data ) {
    try {
      const response = await fetch( si_js_object.rest_url + 'sprout/v1/paypal-checkout/cancel-order', {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify( {
          paypal_order_id: data.orderID,
        } ),
      } );
      if ( response.ok ) {
        alert( 'Payment Cancelled' );
        location.reload();
      } else {
        console.error( response );
      }
    } catch ( error ) {
      console.error( error );
    }
  }
} ).render( '#paypal-button-container' );
