<?php
/**
 * File: sprout-invoices/bundles/sprout-invoices-payments-paypal/si-paypal-checkout.php
 *
 * This file is used to register the Paypal Payment Processor and create the payment button that redirects
 * to a Paypal hosted checkout page.
 *
 * @since 20.6.0
 *
 * @package SI_PayPal
 * @subpackage SI_Offsite_Processors
 */

/**
 * Square Payment Processor
 *
 * This class is used to register the Paypal Payment Processor and create the payment button that redirects
 * to a Paypal hosted checkout page.
 *
 * @since 20.6.0
 *
 * @package SI_PayPal
 * @subpackage SI_Offsite_Processors
 */
class SI_PayPal_Checkout extends SI_Offsite_Processors {
	const MODE_TEST                       = 'test';
	const MODE_LIVE                       = 'live';
	const SI_PAYPAL_CLIENT_ID             = 'si_paypal_client_id';
	const SI_PAYPAL_CLIENT_ID_SANDBOX     = 'si_paypal_client_id_sandbox';
	const SI_PAYPAL_CLIENT_SECRET         = 'si_paypal_client_secret';
	const SI_PAYPAL_CLIENT_SECRET_SANDBOX = 'si_paypal_client_secret_sandbox';
	const SI_PAYPAL_MERCHANT_ID           = 'si_paypal_merchant_id';
	const SI_PAYPAL_CC_OPTION             = 'si_paypal_cc_option';
	const SI_PAYPAL_MODE_OPTION           = 'si_paypal_checkout_mode';
	const CURRENCY_CODE_OPTION            = 'si_square_currency';
	const PAYMENT_METHOD                  = 'PayPal';
	const PAYMENT_SLUG                    = 'paypal-checkout';
	const SI_PAYPAL_ENDPOINT_SANDBOX      = 'https://api-m.sandbox.paypal.com/';
	const SI_PAYPAL_ENDPOINT_LIVE         = 'https://api-m.paypal.com/';

	/**
	 * Instance of this class.
	 *
	 * @since 20.6.0
	 *
	 * @var object
	 */
	protected static $instance;

	/**
	 * API mode.
	 *
	 * Payment Processor can be in test (sandbox) or live (production) mode.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	protected static $api_mode = self::MODE_TEST;

	/**
	 * PayPal Client Id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $si_paypal_client_id;

	/**
	 * PayPal Sandbox Client Id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $si_paypal_client_id_sandbox;

	/**
	 * PayPal Client Secret.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $si_paypal_client_secret;

	/**
	 * PayPal Sandbox Client Secret.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $si_paypal_client_secret_sandbox;

	/**
	 * PayPal Merchant ID.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $si_paypal_merchant_id;

	/**
	 * PayPal Credit Card Processing Option.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $si_paypal_cc_option = 'false';

	/**
	 * Currency code.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $currency_code = 'USD';

	/**
	 * Constructor.
	 *
	 * @since 20.6.0
	 */
	protected function __construct() {
		parent::__construct();

		// Remove pages.
		add_filter( 'si_checkout_pages', array( $this, 'remove_checkout_pages' ) );

		self::$api_mode      = get_option( self::SI_PAYPAL_MODE_OPTION, self::MODE_TEST );
		self::$currency_code = get_option( self::CURRENCY_CODE_OPTION, 'USD' );
	}

	/**
	 * Get the instance of this class.
	 *
	 * @since 20.6.0
	 *
	 * @return object $instance The instance of this class.
	 */
	public static function get_instance() {
		if ( ! ( isset( self::$instance ) && is_a( self::$instance, __CLASS__ ) ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Check if the payment processor is in test mode.
	 *
	 * @since 20.6.0
	 *
	 * @return bool
	 */
	private function is_test() {
		return self::MODE_TEST === self::$api_mode;
	}

	/**
	 * Get type of payment method.
	 *
	 * @since 20.6.0
	 *
	 * @return bool
	 */
	public function get_payment_method() {
		return self::PAYMENT_METHOD;
	}

	/**
	 * Get the payment slug.
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	public function get_slug() {
		return self::PAYMENT_SLUG;
	}

	/**
	 * Get API Url, depending on the mode.
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	private static function get_api_url() {
		if ( self::MODE_LIVE === self::$api_mode ) {
			return self::SI_PAYPAL_ENDPOINT_LIVE;
		} else {
			return self::SI_PAYPAL_ENDPOINT_SANDBOX;
		}
	}

	/**
	 * Get Client ID based on the API Mode.
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	private static function get_client_id() {
		if ( self::MODE_LIVE === self::$api_mode ) {
			return self::$si_paypal_client_id;
		} else {
			return self::$si_paypal_client_id_sandbox;
		}
	}

	/**
	 * Get Client Secret based on the API Mode.
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	private static function get_client_secret() {
		if ( self::MODE_LIVE === self::$api_mode ) {
			return self::$si_paypal_client_secret;
		} else {
			return self::$si_paypal_client_secret_sandbox;
		}
	}

	/**
	 * Register payment processor and options.
	 *
	 * @since 20.6.0
	 *
	 * @return bool
	 */
	public static function register() {

		// Register processor
		self::add_payment_processor( __CLASS__, esc_html__( 'PayPal', 'sprout-invoices' ) );

		if ( ! self::is_active() ) {
			return;
		}

		// Register Options
		self::$si_paypal_client_id             = get_option( self::SI_PAYPAL_CLIENT_ID, '' );
		self::$si_paypal_client_id_sandbox     = get_option( self::SI_PAYPAL_CLIENT_ID_SANDBOX, '' );
		self::$si_paypal_client_secret         = get_option( self::SI_PAYPAL_CLIENT_SECRET, '' );
		self::$si_paypal_client_secret_sandbox = get_option( self::SI_PAYPAL_CLIENT_SECRET_SANDBOX, '' );
		self::$si_paypal_merchant_id           = get_option( self::SI_PAYPAL_MERCHANT_ID, '' );
		self::$si_paypal_cc_option             = get_option( self::SI_PAYPAL_CC_OPTION, 'false' );

		// Resgister API Routes.
		add_action( 'rest_api_init', array( __CLASS__, 'register_api_routes' ) );
	}

	/**
	 * Public name for processor.
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	public static function public_name() {
		return __( 'PayPal', 'sprout-invoices' );
	}

	/**
	 * Hooked on init add the settings page and options.
	 *
	 * @since 20.6.0
	 *
	 * @param  array $settings The settings array.
	 *
	 * @return array $settings The settings array.
	 */
	public static function register_settings( $settings = array() ) {
		// Settings
		$settings['payments'] = array(
			'si_paypal_settings' => array(
				'title'    => esc_html__( 'PayPal Settings', 'sprout-invoices' ),
				'weight'   => 200,
				'settings' => array(
					self::SI_PAYPAL_MODE_OPTION           => array(
						'label'  => esc_html__( 'Mode', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'select',
							'options' => array(
								self::MODE_LIVE => esc_html__( 'Live', 'sprout-invoices' ),
								self::MODE_TEST => esc_html__( 'Sandbox', 'sprout-invoices' ),
							),
							'default' => self::$api_mode,
						),
					),
					self::CURRENCY_CODE_OPTION            => array(
						'label'  => esc_html__( 'Currency Code', 'sprout-invoices' ),
						'option' => array(
							'type'       => 'text',
							'default'    => self::$currency_code,
							'attributes' => array( 'class' => 'small-text' ),
						),
					),
					self::SI_PAYPAL_CLIENT_ID             => array(
						'label'  => esc_html__( 'PayPal Client ID', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$si_paypal_client_id,
						),
					),
					self::SI_PAYPAL_CLIENT_ID_SANDBOX     => array(
						'label'  => esc_html__( 'PayPal Sandbox Client ID', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$si_paypal_client_id_sandbox,
						),
					),
					self::SI_PAYPAL_CLIENT_SECRET         => array(
						'label'  => esc_html__( 'PayPal Client Secret', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$si_paypal_client_secret,
						),
					),
					self::SI_PAYPAL_CLIENT_SECRET_SANDBOX => array(
						'label'  => esc_html__( 'PayPal Sandbox Client Secret', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$si_paypal_client_secret_sandbox,
						),
					),
					self::SI_PAYPAL_MERCHANT_ID           => array(
						'label'  => esc_html__( 'PayPal Merchant ID', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$si_paypal_merchant_id,
						),
					),
					self::SI_PAYPAL_CC_OPTION             => array(
						'label'  => esc_html__( 'Credit Card Processing', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'checkbox',
							'default' => ( 'false' === self::$si_paypal_cc_option ) ? false : true,
							'value'   => 'false',
						),
					),
				),
			),
		);
		return $settings;
	}

	/**
	 * Register API Routes.
	 */
	public static function register_api_routes() {
		register_rest_route(
			'sprout/v1',
			'/paypal-checkout/create-order',
			array(
				'methods'  => 'POST',
				'callback' => array( __CLASS__, 'create_order' ),
			)
		);
		register_rest_route(
			'sprout/v1',
			'/paypal-checkout/capture-order',
			array(
				'methods'  => 'POST',
				'callback' => array( __CLASS__, 'capture_order' ),
			)
		);
		register_rest_route(
			'sprout/v1',
			'/paypal-checkout/cancel-order',
			array(
				'methods'  => 'POST',
				'callback' => array( __CLASS__, 'cancel_payment' ),
			)
		);
	}

	/**
	 * Create an order from invoice.
	 *
	 * @since 20.6.0
	 */
	public static function create_order() {
		$data = json_decode( file_get_contents( 'php://input' ), true );

		$access_token = self::maybe_create_token();

		if ( ! empty( $access_token ) ) {
			// Get the invoice.
			$invoice = SI_Invoice::get_instance( $data['invoice_id'] );
			$fee     = ( SI_Service_Fee::get_service_fee( 'SI_PayPal_Checkout' ) / 100 );
			$total   = $invoice->get_balance() + ( $invoice->get_balance() * $fee );
			$data    = array(
				'intent'         => 'CAPTURE',
				'purchase_units' => array(
					array(
						'invoice_id' => $invoice->get_invoice_id(),
						'amount'     => array(
							'currency_code' => $invoice->get_currency(),
							'value'         => $total,
						),
					),
				),
			);

			$api_url = self::get_api_url() . 'v2/checkout/orders';
			$args    = array(
				'method'  => 'POST',
				'headers' => array(
					'Authorization' => 'Bearer ' . $access_token,
					'Content-Type'  => 'application/json',
					'Prefer'        => 'return=representation',
				),
				'body'    => wp_json_encode( $data ),
			);

			$response              = wp_remote_get( $api_url, $args );
			$paypal_order_response = json_decode( $response['body'] );

			// Create new pending payment for paypal order.
			$si_payment = SI_Payment::new_payment(
				array(
					'payment_method' => self::PAYMENT_METHOD,
					'invoice'        => $paypal_order_response->purchase_units[0]->invoice_id,
					'amount'         => $paypal_order_response->purchase_units[0]->amount->value,
					'data'           => array(
						'invoice'         => $paypal_order_response->purchase_units[0]->invoice_id,
						'paypal_order_id' => $paypal_order_response->id,
						'create_time'     => $paypal_order_response->create_time,
					),
				),
				SI_Payment::STATUS_PENDING
			);

			if ( is_wp_error( $si_payment ) ) {
				wp_send_json_error( 'Pending payment creation failed.' );
			}

			wp_send_json_success( $paypal_order_response->id );
		} else {
			wp_send_json_error( 'Access token creation failed.' );
		}
	}

	/**
	 * Capture an order from PayPal and updated SI_Payment.
	 *
	 * @since 20.6.0
	 */
	public static function capture_order() {
		$data         = json_decode( file_get_contents( 'php://input' ), true );
		$access_token = self::maybe_create_token();

		$api_url = self::get_api_url() . 'v2/checkout/orders/' . $data['paypal_order_id'] . '/capture';
		$args    = array(
			'method'  => 'POST',
			'headers' => array(
				'Authorization' => 'Bearer ' . $access_token,
				'Content-Type'  => 'application/json',
				'Prefer'        => 'return=representation',
			),
		);

		$response                = wp_remote_get( $api_url, $args );
		$paypal_capture_response = json_decode( $response['body'] );

		if ( 'COMPLETED' === $paypal_capture_response->status ) {
			// Update Pending Payment created in create_order().
			$invoice          = SI_Invoice::get_instance( $paypal_capture_response->purchase_units[0]->invoice_id );
			$pending_payments = $invoice->get_pending_payments();
			$pending_payment  = SI_Payment::get_instance( $pending_payments[0] );
			$pending_payment->set_status( SI_Payment::STATUS_COMPLETE );

			wp_send_json_success( $paypal_capture_response );
		}
	}

	/**
	 * Cancel Payment from PayPal.
	 *
	 * @since 20.6.0
	 */
	public static function cancel_payment() {
		$data         = json_decode( file_get_contents( 'php://input' ), true );
		$access_token = self::maybe_create_token();

		$api_url = self::get_api_url() . 'v2/checkout/orders/' . $data['paypal_order_id'];
		$args    = array(
			'method'  => 'GET',
			'headers' => array(
				'Authorization' => 'Bearer ' . $access_token,
				'Content-Type'  => 'application/json',
				'Prefer'        => 'return=representation',
			),
		);

		$response                = wp_remote_get( $api_url, $args );
		$paypal_capture_response = json_decode( $response['body'] );

		$invoice          = SI_Invoice::get_instance( $paypal_capture_response->purchase_units[0]->invoice_id );
		$pending_payments = $invoice->get_pending_payments();
		$pending_payment  = SI_Payment::get_instance( $pending_payments[0] );

		$pending_payment->set_status( SI_Payment::STATUS_CANCELLED );

		wp_send_json_success( $paypal_capture_response );
	}

	/**
	 * Add the payment button to the checkout page
	 *
	 * @since 20.6.0
	 *
	 * @return array $option The checkout options array
	 */
	public static function checkout_options() {
		$option['purchase_button_callback'] = array( __CLASS__, 'payment_button' );
		return $option;
	}

	/**
	 * The review page is unnecessary
	 *
	 * @since 20.6.0
	 *
	 * @param  array $pages The checkout pages.
	 *
	 * @return array $pages The checkout pages.
	 */
	public function remove_checkout_pages( $pages ) {
		unset( $pages[ SI_Checkouts::REVIEW_PAGE ] );
		return $pages;
	}

	/**
	 * Process a payment
	 *
	 * This function is called when the user is redirected back to the site after completing the payment.
	 *
	 * @since 20.6.0
	 *
	 * @param SI_Checkouts $checkout The checkout object.
	 * @param SI_Invoice   $invoice The invoice object.
	 *
	 * @return void
	 */
	public function process_payment( SI_Checkouts $checkout, SI_Invoice $invoice ) {
		// Function is called but not used.
	}

	/**
	 * Load the Payment Button
	 *
	 * @since 20.6.0
	 *
	 * @return void
	 */
	public static function payment_button() {

		// Enqueue the PayPal JS SDK and the custom JS file.
		if ( 'false' === self::$si_paypal_cc_option ) {
			$query_args = '&disable-funding=credit,card,paylater';
		} else {
			$query_args = '&disable-funding=paylater';
		}
		wp_enqueue_script( 'si-paypal-js-sdk', 'https://www.paypal.com/sdk/js?client-id=' . self::get_client_id() . $query_args, array(), null, true ); // phpcs:ignore WordPress.WP.EnqueuedResourceParameters.MissingVersion
		wp_enqueue_script( 'si-paypal', SA_ADDON_PAYPAL_URL . '/resources/si-paypal.js', array( 'jquery' ), self::SI_VERSION, true );

		$view = self::load_addon_view(
			'button',
			array(
				'invoice_id' => get_the_id(),
				'ajax_url'   => get_admin_url() . 'admin-ajax.php',
			),
			true
		);

		print wp_kses_post( $view );
	}

	/**
	 * Check if token for PayPal API already exists.
	 *
	 * @since 20.6.0
	 *
	 * @return string $access_token The access token.
	 */
	public static function maybe_create_token() {
		$access_token = get_transient( 'si_paypal_access_token' );
		if ( ! $access_token || empty( $access_token ) ) {
			$access_token = self::create_token();
			set_transient( 'si_paypal_access_token', $access_token, 3500 );
		}
		return $access_token;
	}

	/**
	 * Create an access token for the PayPal API.
	 *
	 * @since 20.6.0
	 *
	 * @return string $access_token The access token.
	 */
	public static function create_token() {
		$client_id     = self::get_client_id();
		$client_secret = self::get_client_secret();
		$api_url       = self::get_api_url() . 'v1/oauth2/token?grant_type=client_credentials';

		$args = array(
			'method'  => 'POST',
			'headers' => array(
				// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_encode
				'Authorization' => 'Basic ' . base64_encode( $client_id . ':' . $client_secret ),
			),
		);

		$response = wp_remote_get( $api_url, $args );

		if ( is_wp_error( $response ) ) {
			self::set_error_messages( $response->get_error_message() );
			return;
		}
		return json_decode( wp_remote_retrieve_body( $response ) )->access_token;
	}

	/**
	 * Get the currency code for the invoice.
	 *
	 * @since 20.6.0
	 *
	 * @param  int $invoice_id The invoice id.
	 *
	 * @return string $currency_code The currency code.
	 */
	private static function get_currency_code( $invoice_id ) {
		return apply_filters( 'si_currency_code', self::$currency_code, $invoice_id, self::PAYMENT_METHOD );
	}

	/**
	 * Load the addon view.
	 *
	 * In this case it is just the payment button that redirects you to the Square Checkout page.
	 *
	 * @since 20.6.0
	 *
	 * @param  string $view                 The view to load.
	 * @param  array  $args                 Arguments to pass to the view.
	 * @param  bool   $allow_theme_override Whether or not to allow the theme to override the view.
	 *
	 * @return string $view The view to load.
	 */
	public static function load_addon_view( $view, $args, $allow_theme_override = true ) {
		add_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		$view = self::load_view( $view, $args, $allow_theme_override );
		remove_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		return $view;
	}

	/**
	 * Get the path to the addon views
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	public static function addons_view_path() {
		return SA_ADDON_PAYPAL_PATH . '/views/';
	}

	/**
	 * Convert a money value to cents
	 *
	 * @since 20.6.0
	 *
	 * @param  float $value The value to convert.
	 *
	 * @return float $value The value in cents.
	 */
	private static function convert_money_to_cents( $value ) {
		if ( ! apply_filters( 'si_convert_money_to_cents_for_square', true ) ) {
			return (float) $value;
		}
		// strip out commas
		$value = preg_replace( '/\,/i', '', $value );
		// strip out all but numbers, dash, and dot
		$value = preg_replace( '/([^0-9\.\-])/i', '', $value );
		// make sure we are dealing with a proper number now, no +.4393 or 3...304 or 76.5895,94
		if ( ! is_numeric( $value ) ) {
			return 0.00;
		}
		// convert to a float explicitly
		$value = (float) $value;
		return round( $value, 2 ) * 100;
	}

	/**
	 * Convert a cent value to money
	 *
	 * @since 20.6.0
	 *
	 * @param  float $value The value to convert.
	 *
	 * @return float $value The value in money.
	 */
	private static function convert_cents_to_money( $value ) {
		if ( ! apply_filters( 'si_convert_money_to_cents_for_square', true ) ) {
			return (float) $value;
		}
		// strip out commas
		$value = preg_replace( '/\,/i', '', $value );
		// strip out all but numbers, dash, and dot
		$value = preg_replace( '/([^0-9\.\-])/i', '', $value );
		// make sure we are dealing with a proper number now, no +.4393 or 3...304 or 76.5895,94
		if ( ! is_numeric( $value ) ) {
			return 0.00;
		}
		// convert to a float explicitly
		return number_format( floatval( $value / 100 ), 2 );
	}


	/**
	 * Grabs error messages from a Square response and displays them to the user
	 *
	 * @since 20.6.0
	 *
	 * @param string $message The response from Square.
	 * @param bool   $display Whether or not to display the error message.
	 *
	 * @return void
	 */
	private static function set_error_messages( $message, $display = true ) {
		if ( $display ) {
			self::set_message( $message, self::MESSAGE_STATUS_ERROR );
		} else {
			do_action( 'si_error', __CLASS__ . '::' . __FUNCTION__ . ' - error message from square', $message );
		}
	}

}
SI_PayPal_Checkout::register();
