<?php

/**
 * File: sprout-invoices/bundles/sprout-invoices-payments-squareup/square-up.php
 *
 * This file is used to register the Square Payment Processor and create the payment button that redirects
 * to a square hosted checkout page.
 * This file is part of Sprout Invoices, moved from sprout-invoices-payments-squareup/inc/square-up.php.
 *
 * @since 20.6.0
 *
 * @package SA_Square
 * @subpackage SI_Offsite_Processors
 */

require 'inc/vendor/autoload.php';

use Square\SquareClient;
use Square\Environment;
use Square\Exceptions\ApiException;
use Square\Models\CheckoutOptions;
use Square\Models\CreateCheckoutRequest;
use Square\Models\CreateOrderRequest;
use Square\Models\CreatePaymentLinkRequest;
use Square\Models\CreateWebhookSubscriptionRequest;
use Square\Models\Money;
use Square\Models\Order;
use Square\Models\UpdateOrderRequest;
use Square\Models\WebhookSubscription;

/**
 * Square Payment Processor
 *
 * This class is used to register the Square Payment Processor and create the payment button that redirects
 * to a square hosted checkout page.
 *
 * @since 20.6.0
 *
 * @package SA_Square
 * @subpackage SI_Offsite_Processors
 */
class SA_Square extends SI_Offsite_Processors {
	const MODE_TEST                   = 'test';
	const MODE_LIVE                   = 'live';
	const API_SECRET_KEY_OPTION       = 'si_square_access_id';
	const API_SECRET_KEY_TEST_OPTION  = 'si_square_access_id_test';
	const API_PUB_KEY_OPTION          = 'si_square_app_id';
	const API_PUB_KEY_TEST_OPTION     = 'si_square_app_id_test';
	const API_LOCATION_ID_OPTION      = 'si_square_location_id';
	const API_LOCATION_ID_TEST_OPTION = 'si_square_location_id_test';
	const API_MODE_OPTION             = 'si_square_mode';
	const CURRENCY_CODE_OPTION        = 'si_square_currency';
	const PAYMENT_METHOD              = 'Credit (Square)';
	const PAYMENT_SLUG                = 'square';
	const UPDATE                      = 'square_version_upgrade_v1';
	const API_QUERY_VAR               = 'square-sprout-invoices';

	/**
	 * Instance of this class.
	 *
	 * @since 20.6.0
	 *
	 * @var object
	 */
	protected static $instance;

	/**
	 * API mode.
	 *
	 * Payment Processor can be in test (sandbox) or live (production) mode.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	protected static $api_mode = self::MODE_TEST;

	/**
	 * Test API access id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $api_access_id_test;

	/**
	 * Test API app id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $api_app_id_test;

	/**
	 * Test API location id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $api_location_id_test;

	/**
	 * Live API access id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $api_access_id;

	/**
	 * Live API app id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $api_app_id;

	/**
	 * Live API location id.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $api_location_id;

	/**
	 * Currency code.
	 *
	 * @since 20.6.0
	 *
	 * @var string
	 */
	private static $currency_code = 'USD';

	/**
	 * Constructor.
	 *
	 * @since 20.6.0
	 */
	protected function __construct() {
		parent::__construct();
		self::$api_mode      = get_option( self::API_MODE_OPTION, self::MODE_TEST );
		self::$currency_code = get_option( self::CURRENCY_CODE_OPTION, 'USD' );

		// Remove pages.
		add_filter( 'si_checkout_pages', array( $this, 'remove_checkout_pages' ) );

		// Square Checkout Ajax.
		add_filter( 'wp_ajax_si_square_checkout', array( $this, 'si_square_checkout' ) );
		add_action( 'wp_ajax_nopriv_si_square_checkout', array( $this, 'si_square_checkout' ) );
	}

	/**
	 * Get the instance of this class.
	 *
	 * @since 20.6.0
	 *
	 * @return object $instance The instance of this class.
	 */
	public static function get_instance() {
		if ( ! ( isset( self::$instance ) && is_a( self::$instance, __CLASS__ ) ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Check if the payment processor is in test mode.
	 *
	 * @since 20.6.0
	 *
	 * @return bool
	 */
	private function is_test() {
		return self::MODE_TEST === self::$api_mode;
	}

	/**
	 * Get type of payment method.
	 *
	 * @since 20.6.0
	 *
	 * @return bool
	 */
	public function get_payment_method() {
		return self::PAYMENT_METHOD;
	}

	/**
	 * Get the payment slug.
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	public function get_slug() {
		return self::PAYMENT_SLUG;
	}

	/**
	 * Register payment processor and options.
	 *
	 * @since 20.6.0
	 *
	 * @return bool
	 */
	public static function register() {

		// Register processor
		self::add_payment_processor( __CLASS__, esc_html__( 'Square', 'sprout-invoices' ) );

		if ( ! self::is_active() ) {
			return;
		}

		// Register Options
		self::$api_access_id        = get_option( self::API_SECRET_KEY_OPTION, '' );
		self::$api_app_id           = get_option( self::API_PUB_KEY_OPTION, '' );
		self::$api_location_id      = get_option( self::API_LOCATION_ID_OPTION, '' );
		self::$api_access_id_test   = get_option( self::API_SECRET_KEY_TEST_OPTION, '' );
		self::$api_app_id_test      = get_option( self::API_PUB_KEY_TEST_OPTION, '' );
		self::$api_location_id_test = get_option( self::API_LOCATION_ID_TEST_OPTION, '' );

		// Add Square Webhook Subscription.
		self::add_square_webhook();

		// Register query var for Square Webhook Listener.
		self::register_query_var( self::API_QUERY_VAR, array( __CLASS__, 'sq_webhook_listener' ) );
	}

	/**
	 * Public name for processor.
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	public static function public_name() {
		return __( 'Square Checkout', 'sprout-invoices' );
	}

	/**
	 * Add the payment button to the checkout page
	 *
	 * @since 20.6.0
	 *
	 * @return array $option The checkout options array
	 */
	public static function checkout_options() {
		$option['purchase_button_callback'] = array( __CLASS__, 'payment_button' );
		return $option;
	}

	/**
	 * The review page is unnecessary
	 *
	 * @since 20.6.0
	 *
	 * @param  array $pages The checkout pages.
	 *
	 * @return array $pages The checkout pages.
	 */
	public function remove_checkout_pages( $pages ) {
		unset( $pages[ SI_Checkouts::REVIEW_PAGE ] );
		return $pages;
	}

	/**
	 * Hooked on init add the settings page and options.
	 *
	 * @since 20.6.0
	 *
	 * @param  array $settings The settings array.
	 *
	 * @return array $settings The settings array.
	 */
	public static function register_settings( $settings = array() ) {
		// Settings
		$settings['payments'] = array(
			'si_square_settings' => array(
				'title'    => esc_html__( 'Square Settings', 'sprout-invoices' ),
				'weight'   => 200,
				'settings' => array(
					self::API_MODE_OPTION             => array(
						'label'  => esc_html__( 'Mode', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'select',
							'options' => array(
								self::MODE_LIVE => esc_html__( 'Live', 'sprout-invoices' ),
								self::MODE_TEST => esc_html__( 'Sandbox', 'sprout-invoices' ),
							),
							'default' => self::$api_mode,
						),
					),
					self::CURRENCY_CODE_OPTION        => array(
						'label'  => esc_html__( 'Currency Code', 'sprout-invoices' ),
						'option' => array(
							'type'       => 'text',
							'default'    => self::$currency_code,
							'attributes' => array( 'class' => 'small-text' ),
						),
					),
					self::API_SECRET_KEY_OPTION       => array(
						'label'  => esc_html__( 'Personal Access Token', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_access_id,
						),
					),
					self::API_PUB_KEY_OPTION          => array(
						'label'  => esc_html__( 'Application ID', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_app_id,
						),
					),
					self::API_LOCATION_ID_OPTION      => array(
						'label'  => esc_html__( 'Location ID', 'sprout-invoices' ),
						'option' => array(
							'type'        => 'text',
							'default'     => self::$api_location_id,
							'description' => esc_html__( 'Settings->App integrations->Manage->Locations', 'sprout-invoices' ),
						),
					),
					self::API_SECRET_KEY_TEST_OPTION  => array(
						'label'  => esc_html__( 'Sandbox Personal Access Token', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_access_id_test,
						),
					),
					self::API_PUB_KEY_TEST_OPTION     => array(
						'label'  => esc_html__( 'Sandbox Application ID', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_app_id_test,
						),
					),
					self::API_LOCATION_ID_TEST_OPTION => array(
						'label'  => esc_html__( 'Location ID', 'sprout-invoices' ),
						'option' => array(
							'type'        => 'text',
							'default'     => self::$api_location_id_test,
							'description' => esc_html__( 'Settings->App integrations->Manage->Locations', 'sprout-invoices' ),
						),
					),
				),
			),
		);
		return $settings;
	}


	/**
	 * Load the Payment Button
	 *
	 * @since 20.6.0
	 *
	 * @return void
	 */
	public static function payment_button() {
		wp_enqueue_script( 'si-square', SA_ADDON_SQUARE_URL . '/resources/square.jquery.js', array( 'jquery' ), '1.0.1', true );
		wp_enqueue_style( 'si-square', SA_ADDON_SQUARE_URL . '/resources/square-sprout.css', array(), '1.0.1' );
		$view = self::load_addon_view(
			'button',
			array(
				'invoice_id' => get_the_id(),
				'ajax_url'   => get_admin_url() . 'admin-ajax.php',
			),
			true
		);

		print wp_kses_post( $view );
	}

	/**
	 * Load Square SDK and create client object
	 *
	 * @since 20.6.0
	 *
	 * @return void|SquareClient Square Client Object.
	 */
	public static function get_square_client() {
		$access_id = self::MODE_TEST === get_option( self::API_MODE_OPTION, self::MODE_TEST ) ?
			self::$api_access_id_test :
			self::$api_access_id;

		$square_mode = self::MODE_TEST === get_option( self::API_MODE_OPTION, self::MODE_TEST ) ? 'sandbox' : 'production';

		if ( '' === $access_id ) {
			return;
		}

		return new SquareClient(
			array(
				'accessToken' => $access_id,
				'environment' => $square_mode,
			)
		);

	}

	/**
	 * Add Square Webhook Subscriptions.
	 *
	 * @since 20.6.0
	 *
	 * @return void
	 */
	public static function add_square_webhook() {

		$client = self::get_square_client();

		if ( ! $client ) {
			return;
		}

		$maybe_webhook_set = self::check_webhook_subscription( $client );

		if ( $maybe_webhook_set ) {
			return;
		}

		$webhook_subscription = new WebhookSubscription();
		$webhook_subscription->setName( 'Sprout Invoices Webhook' );
		$webhook_subscription->setEventTypes( array( 'payment.created', 'payment.updated', 'order.updated' ) );
		$webhook_subscription->setNotificationUrl( site_url() . '?square-sprout-invoices=callback' );

		$webhook_request = new CreateWebhookSubscriptionRequest( $webhook_subscription );
		$webhook_request->setIdempotencyKey( uniqid() );

		$api_response = $client->getWebhookSubscriptionsApi()->createWebhookSubscription( $webhook_request );

		if ( $api_response->isSuccess() ) {
			$result = $api_response->getResult();
		} else {
			$result = $api_response->getErrors();
		}
	}


	/**
	 * Check if the Webhook Subscription is already set for Sprout Invoices.
	 *
	 * @since 20.6.0
	 *
	 * @param object $client Square Client Object.
	 *
	 * @return bool  True if the Webhook Subscription is set, false if not.
	 */
	public static function check_webhook_subscription( $client ) {

		if ( ! $client ) {
			return;
		}

		$webhook_subscriptions = $client->getWebhookSubscriptionsApi()->listWebhookSubscriptions();

		if ( $webhook_subscriptions->isSuccess() ) {
			$result = $webhook_subscriptions->getResult()->getSubscriptions();
		} else {
			$error = $webhook_subscriptions->getErrors();
			return true;
		}

		// Check for Duplicate Sprout Invoices Webhook Subscription URLs.
		if ( ! empty( $result ) ) {
			foreach ( $result as $subscription ) {
				if ( $subscription->getNotificationUrl() === get_site_url() . '?square-sprout-invoices=callback' ) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Create Square Webhook Listener.
	 *
	 * @since 20.6.0
	 *
	 * @return void
	 */
	public static function sq_webhook_listener() {
		// Check if the request is a Square Webhook.
		if ( ! isset( $_REQUEST[ self::API_QUERY_VAR ] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			return;
		}

		$webhook_body = file_get_contents( 'php://input' );
		$webhook_body = json_decode( $webhook_body, true );
		$webhook_type = $webhook_body['type'];

		if ( 'payment.updated' === $webhook_type ) {
			$payment_id     = $webhook_body['data']['id'];
			$square_payment = self::get_square_client()->getPaymentsApi()->getPayment( $payment_id )->getResult()->getPayment();
			$order_id       = $webhook_body['data']['object']['payment']['order_id'];
			$invoice_id     = self::get_invoice_id( $order_id );
			$invoice        = SI_Invoice::get_instance( $invoice_id );

			// Check if invoice exists.
			if ( ! $invoice ) {
				return;
			}

			// Check Invoice balance.
			if ( empty( $invoice->get_balance() ) ) {
				return;
			}

			// Create SI Payment Object.
			$payment = SI_Payment::new_payment(
				array(
					'payment_method' => self::PAYMENT_METHOD,
					'invoice'        => $invoice_id,
					'amount'         => self::convert_cents_to_money( $square_payment->getAmountMoney()->getAmount() ),
					'data'           => array(
						'invoice'         => $invoice_id,
						'square_order_id' => $order_id,
						'api_response'    => $webhook_body['data']['object']['payment'],
					),
				),
				SI_Payment::STATUS_COMPLETE
			);
		}

	}

	/**
	 * Create Square Checkout session.
	 *
	 * @since 20.6.0
	 *
	 * @return void
	 */
	public function si_square_checkout() {
		$nonce = isset( $_POST['data']['nonce'] ) ? sanitize_text_field( wp_unslash( $_POST['data']['nonce'] ) ) : '';

		if ( ! wp_verify_nonce( $nonce, 'sprout_invoices_controller_nonce' ) ) {
			wp_send_json_error( array( 'error' => 'Invalid Nonce' ), 403 );
		}

		$sq_location_id = self::MODE_TEST === get_option( self::API_MODE_OPTION, self::MODE_TEST ) ?
			self::$api_location_id_test :
			self::$api_location_id;

		$invoice_id    = $_POST['data']['invoice_id'];
		$invoice       = SI_Invoice::get_instance( $invoice_id );
		$user          = si_who_is_paying( $invoice );
		$user_email    = $user ? $user->user_email : '';
		$square_client = self::get_square_client();
		$line_items    = $invoice->get_line_items();

		$invoice_amount = new \Square\Models\Money();
		$invoice_amount->setAmount( self::convert_money_to_cents( $invoice->get_balance() ) );
		$invoice_amount->setCurrency( self::get_currency_code( $invoice_id ) );

		$order = new \Square\Models\QuickPay(
			'Invoice - ' . esc_html( $invoice_id ),
			$invoice_amount,
			$sq_location_id
		);

		// Add Checkout options.
		$checkout_options = new \Square\Models\CheckoutOptions();
		$checkout_options->setRedirectUrl( get_permalink( $invoice->get_id() ) );

		// Create Payment request.
		$create_payment_request = new CreatePaymentLinkRequest();
		$create_payment_request->setIdempotencyKey( uniqid() );
		$create_payment_request->setQuickPay( $order );
		$create_payment_request->setCheckoutOptions( $checkout_options );

		// Get the checkout link from Checkout API.
		$api_response = $square_client->getCheckoutApi()->createPaymentLink( $create_payment_request );

		if ( $api_response->isSuccess() ) {
			$result = $api_response->getResult();
			$order  = $result->getRelatedResources()->getOrders()[0]->getId();

			// Create a new order object.
			$new_order = new Order( $sq_location_id );
			$new_order->setReferenceId( $invoice_id );
			$new_order->setVersion( 1 );

			// Update the order with the invoice id.
			$update_order = new UpdateOrderRequest();
			$update_order->setOrder( $new_order );
			$square_client->getOrdersApi()->updateOrder( $order, $update_order );
		} else {
			$result = $api_response->getErrors();
		}

		wp_send_json_success( $result );
	}

	/**
	 * Get Invoice id from Square Order.
	 *
	 * @since 20.6.0
	 *
	 * @param  string $order_id Square Order ID.
	 *
	 * @return string $invoice_id Sprout Invoice ID.
	 */
	public static function get_invoice_id( $order_id ) {

		return self::get_square_client()
			->getOrdersApi()
			->retrieveOrder( $order_id )
			->getResult()
			->getOrder()
			->getReferenceId();
	}

	/**
	 * Process a payment
	 *
	 * This function is called when the user is redirected back to the site after completing the payment.
	 *
	 * @since 20.6.0
	 *
	 * @param SI_Checkouts $checkout The checkout object.
	 * @param SI_Invoice   $invoice The invoice object.
	 *
	 * @return void
	 */
	public function process_payment( SI_Checkouts $checkout, SI_Invoice $invoice ) {
		// Function is called but not used.
	}


	/**
	 * Get the currency code for the invoice.
	 *
	 * @since 20.6.0
	 *
	 * @param  int $invoice_id The invoice id.
	 *
	 * @return string $currency_code The currency code.
	 */
	private static function get_currency_code( $invoice_id ) {
		return apply_filters( 'si_currency_code', self::$currency_code, $invoice_id, self::PAYMENT_METHOD );
	}

	/**
	 * Load the addon view.
	 *
	 * In this case it is just the payment button that redirects you to the Square Checkout page.
	 *
	 * @since 20.6.0
	 *
	 * @param  string $view                 The view to load.
	 * @param  array  $args                 Arguments to pass to the view.
	 * @param  bool   $allow_theme_override Whether or not to allow the theme to override the view.
	 *
	 * @return string $view The view to load.
	 */
	public static function load_addon_view( $view, $args, $allow_theme_override = true ) {
		add_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		$view = self::load_view( $view, $args, $allow_theme_override );
		remove_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		return $view;
	}

	/**
	 * Get the path to the addon views
	 *
	 * @since 20.6.0
	 *
	 * @return string
	 */
	public static function addons_view_path() {
		return SA_ADDON_SQUARE_PATH . '/views/';
	}

	/**
	 * Convert a money value to cents
	 *
	 * @since 20.6.0
	 *
	 * @param  float $value The value to convert.
	 *
	 * @return float $value The value in cents.
	 */
	private static function convert_money_to_cents( $value ) {
		if ( ! apply_filters( 'si_convert_money_to_cents_for_square', true ) ) {
			return (float) $value;
		}
		// strip out commas
		$value = preg_replace( '/\,/i', '', $value );
		// strip out all but numbers, dash, and dot
		$value = preg_replace( '/([^0-9\.\-])/i', '', $value );
		// make sure we are dealing with a proper number now, no +.4393 or 3...304 or 76.5895,94
		if ( ! is_numeric( $value ) ) {
			return 0.00;
		}
		// convert to a float explicitly
		$value = (float) $value;
		return round( $value, 2 ) * 100;
	}

	/**
	 * Convert a cent value to money
	 *
	 * @since 20.6.0
	 *
	 * @param  float $value The value to convert.
	 *
	 * @return float $value The value in money.
	 */
	private static function convert_cents_to_money( $value ) {
		if ( ! apply_filters( 'si_convert_money_to_cents_for_square', true ) ) {
			return (float) $value;
		}
		// strip out commas
		$value = preg_replace( '/\,/i', '', $value );
		// strip out all but numbers, dash, and dot
		$value = preg_replace( '/([^0-9\.\-])/i', '', $value );
		// make sure we are dealing with a proper number now, no +.4393 or 3...304 or 76.5895,94
		if ( ! is_numeric( $value ) ) {
			return 0.00;
		}
		// convert to a float explicitly
		return number_format( floatval( $value / 100 ), 2 );
	}


	/**
	 * Grabs error messages from a Square response and displays them to the user
	 *
	 * @since 20.6.0
	 *
	 * @param string $message The response from Square.
	 * @param bool   $display Whether or not to display the error message.
	 *
	 * @return void
	 */
	private function set_error_messages( $message, $display = true ) {
		if ( $display ) {
			self::set_message( $message, self::MESSAGE_STATUS_ERROR );
		} else {
			do_action( 'si_error', __CLASS__ . '::' . __FUNCTION__ . ' - error message from square', $message );
		}
	}
}
SA_Square::register();
