<?php

class SI_Stripe_Checkout extends SI_Offsite_Processors {
	const MODE_TEST = 'test';
	const MODE_LIVE = 'live';

	const API_SECRET_KEY_OPTION      = 'si_stripe_secret_key';
	const API_SECRET_KEY_TEST_OPTION = 'si_stripe_secret_key_test';
	const API_PUB_KEY_OPTION         = 'si_stripe_pub_key';
	const API_PUB_KEY_TEST_OPTION    = 'si_stripe_pub_key_test';

	const CANCEL_URL_OPTION = 'si_stripe_cancel_url';
	const RETURN_URL_OPTION = 'si_stripe_return_url';

	const STRIPE_CHECKOUT_CUSTOMER_KEY_USER_META = 'si_stripe_customer_id_v1';

	const API_MODE_OPTION      = 'si_stripe_mode';
	const CURRENCY_CODE_OPTION = 'si_stripe_currency';
	const BILLING_TYPE_OPTION  = 'si_stripe_billing_type';

	const PAYMENT_METHOD = 'Credit (Stripe Checkout)';
	const PAYMENT_SLUG   = 'stripe_checkout';

	// webhooks
	const API_QUERY_VAR = 'si_stripe_checkout';

	protected static $instance;
	protected static $api_mode = self::MODE_TEST;

	private static $api_secret_key;
	private static $api_secret_key_test;
	private static $api_pub_key;
	private static $api_pub_key_test;
	private static $cancel_url = '';

	/**
	 * Currency code.
	 *
	 * @var string
	 */
	private static $currency_code;

	/**
	 * Billing type.
	 *
	 * @var string
	 */
	private static $billing_type = 'false';


	/**
	 * Get instance of the class.
	 *
	 * @return SI_Stripe_Checkout $instance Stipe checkout class instance.
	 */
	public static function get_instance() {
		if ( ! ( isset( self::$instance ) && is_a( self::$instance, __CLASS__ ) ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Set mode of API to testing.
	 *
	 * @return string $api_mode set mode to 'test'.
	 */
	private static function is_test() {
		return self::MODE_TEST === self::$api_mode;
	}

	/**
	 * Get the payment method.
	 *
	 * @return string payment method.
	 */
	public function get_payment_method() {
		return self::PAYMENT_METHOD;
	}

	/**
	 * Get the payment type.
	 *
	 * @return string payment type.
	 */
	public function get_payment_type() {
		return self::PAYMENT_TYPE;
	}

	/**
	 * Get the payment slug.
	 *
	 * @return string payment slug.
	 */
	public function get_slug() {
		return self::PAYMENT_SLUG;
	}

	/**
	 * Get the payment slug returned from offsite checkout.
	 *
	 * @return string payment slug.
	 */
	public static function returned_from_offsite() {
		return ( isset( $_GET[ self::PAYMENT_SLUG ] ) );
	}

	/**
	 * Register the payment processor and add the necessary hooks.
	 *
	 * @return void if payment processor is not active.
	 */
	public static function register() {
		// Register processor.
		self::add_payment_processor( __CLASS__, __( 'Stripe Checkout', 'sprout-invoices' ) );

		// Check if Payment Processor is active.
		if ( ! self::is_active() ) {
			return;
		}

		// Register Checkout Ajax.
		add_action( 'wp_ajax_si_stripe_checkout', array( __CLASS__, 'stripe_checkout_session' ) );
		add_action( 'wp_ajax_nopriv_si_stripe_checkout', array( __CLASS__, 'stripe_checkout_session' ) );

		// Register Stripe API Authentication.
		add_filter( 'si_api_authentication', array( __CLASS__, 'authenticate_stripe' ) );

		// Register webhooks.
		self::register_query_var( self::PAYMENT_SLUG, array( __CLASS__, 'maybe_back_from_stripe' ) );
		self::register_query_var( self::API_QUERY_VAR, array( __CLASS__, 'webhook_listener' ) );

		// Add Recurring button.
		add_action( 'recurring_payments_profile_info', array( __CLASS__, 'stripe_profile_link' ) );

	}

	/**
	 * Authenticate with Stripe
	 *
	 * @return string $authentication If API key is not valid, return error message
	 */
	public static function authenticate_stripe() {
		if ( ! self::is_active() ) {
			return;
		}
		if ( ! class_exists( 'Stripe' ) ) {
			require_once 'inc/sdk/init.php';
		}
		if ( 'test' === self::$api_mode ) {
			try {
				self::$api_secret_key_test = get_option( self::API_SECRET_KEY_TEST_OPTION, '' );
				if ( ! empty( self::$api_secret_key_test ) ) {
					$stripe = new \Stripe\StripeClient( self::$api_secret_key_test );
				}
				$test_auth = $stripe->customers->all( ['limit' => 3] );
			} catch ( Exception $e ) {
				$authentication = $e->getMessage();
				return $authentication;
			}
		} else {
			try {
				self::$api_secret_key = get_option( self::API_SECRET_KEY_OPTION, '' );
				if ( ! empty( self::$api_secret_key ) ) {
					$stripe = new \Stripe\StripeClient( self::$api_secret_key );
				}
				$test_auth = $stripe->customers->all( ['limit' => 3] );
			} catch ( Exception $e ) {
				$authentication = $e->getMessage();
				return $authentication;
			}
		}
		self::maybe_create_hook_endpoint();
	}

	/**
	 * Create public name for payment method.
	 *
	 * @return string Public name for payment method.
	 */
	public static function public_name() {
		return __( 'Credit Card', 'sprout-invoices' );
	}

	/**
	 * Create checkout option that is added to the invoice.
	 *
	 * @return array $option Checkout option.
	 */
	public static function checkout_options() {
		$option['purchase_button_callback'] = array( __CLASS__, 'payment_button' );
		return $option;
	}

	/**
	 * Constructor.
	 */
	protected function __construct() {
		parent::__construct();
		self::$api_mode           = get_option( self::API_MODE_OPTION, self::MODE_TEST );
		self::$billing_type       = get_option( self::BILLING_TYPE_OPTION, 'false' );

		self::$api_secret_key      = get_option( self::API_SECRET_KEY_OPTION, '' );
		self::$api_secret_key_test = get_option( self::API_SECRET_KEY_TEST_OPTION, '' );
		self::$api_pub_key         = get_option( self::API_PUB_KEY_OPTION, '' );
		self::$api_pub_key_test    = get_option( self::API_PUB_KEY_TEST_OPTION, '' );
		self::$currency_code       = get_option( self::CURRENCY_CODE_OPTION, 'USD' );

		self::$cancel_url = get_option( self::CANCEL_URL_OPTION, add_query_arg( array( 'stripe_checkout' => 'cancelled' ), home_url( '/' ) ) );
		if ( self::$cancel_url === '' ) {
			$url = add_query_arg( array( 'stripe_checkout' => 'cancelled' ), home_url( '/' ) );
			update_option( self::CANCEL_URL_OPTION, $url );
			self::$cancel_url = esc_url_raw( $url );
		}

	}


	/**
	 * Hooked on init add the settings page and options.
	 *
	 * @param  array $settings settings array.
	 * @return array $settings settings array.
	 */
	public static function register_settings( $settings = array() ) {
		// Settings
		$settings['payments'] = array(
			'si_stripe_checkout_settings' => array(
				'title'    => __( 'Stripe Checkout Settings', 'sprout-invoices' ),
				'weight'   => 200,
				'settings' => array(
					self::API_MODE_OPTION            => array(
						'label'  => __( 'Mode', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'select',
							'options' => array(
								self::MODE_LIVE => __( 'Live', 'sprout-invoices' ),
								self::MODE_TEST => __( 'Test', 'sprout-invoices' ),
							),
							'default' => self::$api_mode,
						),
					),
					self::API_SECRET_KEY_OPTION      => array(
						'label'  => __( 'Live Secret Key', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_secret_key,
						),
					),
					self::API_PUB_KEY_OPTION         => array(
						'label'  => __( 'Live Publishable Key', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_pub_key,
						),
					),
					self::API_SECRET_KEY_TEST_OPTION => array(
						'label'  => __( 'Test Secret Key', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_secret_key_test,
						),
					),
					self::API_PUB_KEY_TEST_OPTION    => array(
						'label'  => __( 'Test Publishable Key', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'text',
							'default' => self::$api_pub_key_test,
						),
					),
					self::CURRENCY_CODE_OPTION        => array(
						'label'  => __( 'Currency Code', 'sprout-invoices' ),
						'option' => array(
							'type'       => 'text',
							'default'    => self::$currency_code,
							'attributes' => array( 'class' => 'small-text' ),
						),
					),
					self::BILLING_TYPE_OPTION          => array(
						'label'  => __( 'Enable Address Collection', 'sprout-invoices' ),
						'option' => array(
							'type'    => 'checkbox',
							'default' => ( 'false' === self::$billing_type ) ? false : true,
							'value'   => 'false',
						),
					),
				),
			),
		);
		return $settings;
	}

	/**
	 * Load the Stripe SDK
	 *
	 * @return void|boolean return false if the SDK load errors.
	 */
	private static function load_sdk() {
		if ( ! class_exists( 'Stripe' ) ) {
			require_once 'inc/sdk/init.php';
		} else {
			do_action( 'si_error', __CLASS__ . '::' . __FUNCTION__ . ' - the Stripe class is already included.', null );
		}
		try {
			// Setup the API
			$key = ( self::is_test() ) ? self::$api_secret_key_test : self::$api_secret_key;

			\Stripe\Stripe::setAppInfo(
				'Sprout Invoices',
				self::SI_VERSION,
				'https://sproutinvoices.com/sprout-invoices/',
				'pp_partner_DL3bDXVUCYCNPQ'
			);

			\Stripe\Stripe::setApiKey( $key );
		} catch ( Exception $e ) {
			self::set_error_messages( $e->getMessage() );
			return false;
		}
	}

	/**
	 * Load the Payment Button
	 */
	public static function payment_button() {
		wp_enqueue_script( 'si_stripe_checkout_js', plugins_url( 'resources/si-stripe.js', __FILE__ ), array( 'jquery' ), self::SI_VERSION, true );
		wp_enqueue_script( 'si_stripe_checkout_button_js', 'https://js.stripe.com/v3/', array( 'jquery' ), self::SI_VERSION, true );

		$view = self::load_addon_view(
			'button',
			array(
				'invoice_id' => get_the_id(),
				'ajax_url'   => get_admin_url() . 'admin-ajax.php',
			),
			true
		);
		print $view;
	}

	/**
	 * Create a Stripe checkout session from an invoice (Ajax).
	 *
	 * @since 20.3
	 */
	public static function stripe_checkout_session() {
		$nonce = isset( $_POST['data']['nonce'] ) ? sanitize_text_field( wp_unslash( $_POST['data']['nonce'] ) ) : '';

		if ( ! wp_verify_nonce( $nonce, 'sprout_invoices_controller_nonce' ) ) {
			wp_send_json_error( array( 'error' => 'Invalid Nonce' ), 403 );
		}

		if ( ! isset( $_POST['data']['invoice_id'] ) ) {
			wp_send_json_error( __( 'Invoice ID not set.', 'sprout-invoices' ) );
		}

		$invoice_id     = $_POST['data']['invoice_id'];
		$invoice        = SI_Invoice::get_instance( $invoice_id );
		$payment_amount = ( si_has_invoice_deposit( $invoice->get_id() ) ) ? $invoice->get_deposit() : $invoice->get_balance();
		$user           = si_who_is_paying( $invoice );
		$user_email     = ( $user ) ? $user->user_email : '';

		$billing_type_option = get_option( self::BILLING_TYPE_OPTION );
		if ( 'true' === $billing_type_option ) {
			$billing_type = 'required';
		} else {
			$billing_type = 'auto';
		}

		self::load_sdk();

		$stripe = new \Stripe\StripeClient(
			( self::is_test() ) ? self::$api_secret_key_test : self::$api_secret_key
		);

		$payment_id = self::check_for_old_sessions( $invoice );

		if ( ! $payment_id ) {
			// create a pending payment as the checkout session
			$payment_id = SI_Payment::new_payment(
				array(
					'payment_method'             => self::PAYMENT_METHOD,
					'invoice'                    => $invoice->get_id(),
					'amount'                     => $payment_amount,
					'billing_address_collection' => $billing_type,
					'data'                       => array(
						'live'         => ( self::MODE_LIVE === self::$api_mode ),
						'api_response' => 'Stripe Checkout',
					),
				),
				SI_Payment::STATUS_VOID
			);
		}

		// Create coupon for previous payments and discounts.
		$coupon_total = self::calculate_coupon_total( $invoice );

		if ( $coupon_total > 0 ) {
			$coupon = $stripe->coupons->create(
				array(
					'duration'   => 'once',
					'amount_off' => self::convert_money_to_cents( $coupon_total, $invoice ),
					'currency'   => self::get_currency_code( $invoice_id ),
					'name'       => esc_html__( 'previous payments and discounts.', 'sprout-invoices' ),
				)
			);
		}

		if ( si_is_invoice_recurring( $invoice ) ) {

				$plan_id     = self::create_recurring_payment_plan( $invoice );
				$description = "";

				$create = array(
					'client_reference_id'  => $payment_id,
					'customer_email'       => $user_email,
					'payment_intent_data'        => array(
						'description' => apply_filters( 'si_stripe_checkout_add_description', sprintf( __('%1$s : %2$s', 'sprout-invoices' ),  $user->data->user_login, $invoice->get_invoice_id() ) , $description ),
					),
					'payment_method_types' => array( $payment_type ),
					'subscription_data'    => array(
						'items' => array(
							array(
								'plan' => $plan_id,
							),
						),
					),
					'success_url'          => add_query_arg( array( self::PAYMENT_SLUG => $payment_id ), get_permalink( $invoice_id ) ),
					'cancel_url'           => add_query_arg(
						array(
							self::PAYMENT_SLUG => $payment_id,
							'cancelled'        => 'true',
						),
						get_permalink( $invoice_id )
					),
				);

		} else {

			$session_line_items = self::get_line_item_array( $invoice );
			$description        = "";

			$create = array(
				'client_reference_id'        => $payment_id,
				'customer_email'             => $user_email,
				'payment_intent_data'        => array(
					'description' => apply_filters( 'si_stripe_checkout_add_description', sprintf( __('%1$s : %2$s', 'sprout-invoices' ),  $user->data->user_login, $invoice->get_invoice_id() ) , $description ),
				),
				'line_items'                 => $session_line_items,
				'billing_address_collection' => $billing_type,
				'mode'                       => 'payment',
				'success_url'                => add_query_arg( array( self::PAYMENT_SLUG => $payment_id ), get_permalink( $invoice_id ) ),
				'cancel_url'                 => add_query_arg(
					array(
						self::PAYMENT_SLUG => $payment_id,
						'cancelled'        => 'true',
					),
					get_permalink( $invoice_id )
				),
			);

			if ( isset( $coupon ) && ! empty( $coupon ) ) {
				$create['discounts'] = array(
					array(
						'coupon' => $coupon->id,
					),
				);
			}
		}

		if ( 'Deposit' === $create['line_items'][0]['price_data']['product_data']['name'] ) {
			$depost_payment = SI_Payment::get_instance( $payment_id );
			$depost_payment->set_type( 'deposit' );
			unset( $create['discounts'] );
		}

		$result = \Stripe\Checkout\Session::create(
			array(
				$create,
			)
		);

		wp_send_json_success( $result );

	}

	/**
	 * Check for old Stripe sessions.
	 *
	 * @param  SI_Invoice $invoice invoice object.
	 * @return int
	 */
	public static function check_for_old_sessions( SI_Invoice $invoice ) {

		$invoice_id     = $invoice->get_id();
		$payment_amount = ( si_has_invoice_deposit( $invoice->get_id() ) ) ? $invoice->get_deposit() : $invoice->get_balance();

		$args = array(
			'post_type'        => SI_Payment::POST_TYPE,
			'post_status'      => 'void',
			'meta_key'         => '_payment_invoice',
			'meta_value'       => $invoice_id,
			'posts_per_page'   => -1,
			'fields'           => 'ids',
			'si_bypass_filter' => true,
		);
		$result = get_posts( $args );

		if ( ! is_array( $result ) || empty( $result ) ) {
			return false;
		}

		foreach ( $result as $payment_id ) {
			$payment             = SI_Payment::get_instance( $payment_id );
			$reusable_payment_id = $payment_id;

			if ( $payment->get_payment_method() !== self::PAYMENT_METHOD ) {
				self::delete_payment( $payment_id );
				$reusable_payment_id = false;
				continue;
			}

			if ( $payment_amount !== $payment->get_amount() ) {
				self::delete_payment( $payment_id );
				$reusable_payment_id = false;
				continue;
			}
		}

		return $reusable_payment_id;
	}

	/**
	 * Process a payment
	 *
	 * @param   SI_Checkouts $checkout Checkout object.
	 * @param   SI_Invoice   $invoice Invoice object.
	 */
	public function process_payment( SI_Checkouts $checkout, SI_Invoice $invoice ) {
		// unused abstract function is created in SI_Payment_Processors. This is a placeholder.
	}

	/**
	 * If the user is returning from Stripe with the cancel url display a message that the payment is cancelled.
	 *
	 * @return boolean      false if not returning from Stripe or payment was cancelled.
	 */
	public static function maybe_back_from_stripe() {
		if ( self::returned_from_offsite() && ! is_numeric( $_GET[ self::PAYMENT_SLUG ] ) ) {
			return false;
		}

		$payment_id = $_GET[ self::PAYMENT_SLUG ];

		if ( isset( $_GET['cancelled'] ) && 'true' == $_GET['cancelled']  ) {

			self::set_message( __( 'Payment Cancelled', 'sprout-invoices' ), self::MESSAGE_STATUS_INFO );

			self::delete_payment( $payment_id );

			wp_redirect( remove_query_arg( array( self::PAYMENT_SLUG, 'cancelled' ) ) );
			exit();
		}

		$payment = SI_Payment::get_instance( $payment_id );

		if ( ! is_a( $payment, 'SI_Payment' ) ) {
			self::set_message( __( 'Payment Invalid', 'sprout-invoices' ), self::MESSAGE_STATUS_INFO );
			wp_redirect( remove_query_arg( self::PAYMENT_SLUG ) );
			exit();
		}

		$invoice_id = $payment->get_invoice_id();
		$invoice = SI_Invoice::get_instance( $invoice_id );

		// Messaging
		if ( $payment->get_status() === SI_Payment::STATUS_PENDING ) {
			if ( $invoice->get_balance() < 0.01 ) {
				self::set_message( __( 'Payment Pending', 'sprout-invoices' ), self::MESSAGE_STATUS_INFO );
			} else {
				self::set_message( sprintf( __( 'Pending Payment Received. Current Balance: %s', 'sprout-invoices' ), sa_get_formatted_money( $invoice->get_balance() ) ), self::MESSAGE_STATUS_INFO );
			}
		} else {
			if ( $invoice->get_balance() < 0.01 ) {
				self::set_message( __( 'Payment Received & Invoice Paid!', 'sprout-invoices' ), self::MESSAGE_STATUS_INFO );
			} else {
				self::set_message( sprintf( __( 'Partial Payment Received. Current Balance: %s', 'sprout-invoices' ), sa_get_formatted_money( $invoice->get_balance() ) ), self::MESSAGE_STATUS_INFO );
			}
		}

		// Send new payment notification manually since it was not sent when payment object was originally created.
		do_action( 'si_new_payment', $payment, $args = array() );

		wp_redirect( remove_query_arg( self::PAYMENT_SLUG ) );
		exit();

	}

	/**
	 * Complete the payment in Sprout Invoices if the payment was successful.
	 *
	 * @param  int   $payment_id Payment ID.
	 * @param  array $data Payment data.
	 * @return boolean true if payment was completed.
	 */
	public static function complete_payment( $payment_id, $data ) {

		$payment = SI_Payment::get_instance( $payment_id );
		if ( ! is_a( $payment, 'SI_Payment' ) ) {
			return false;
		}

		if ( $payment->get_payment_method() !== self::PAYMENT_METHOD ) {
			return false;
		}

		// already updated
		if ( $payment->get_status() === SI_Payment::STATUS_COMPLETE ) {
			return true;
		}

		$payment->set_data( $data );

		do_action( 'payment_authorized', $payment );
		$payment->set_status( SI_Payment::STATUS_COMPLETE );
		do_action( 'payment_complete', $payment );
		return true;
	}

	/**
	 * Delete the payment.
	 *
	 * @param  int $payment_id Payment ID.
	 * @return boolean true if payment was deleted.
	 */
	public static function delete_payment( $payment_id ) {
		wp_delete_post( $payment_id, true );
		return true;
	}

	/**
	 * Get the basic line item array for a Stripe payment.
	 *
	 * @param  SI_Invoice $invoice Invoice object.
	 * @return array  $session_line_items Stripe line item array.
	 */
	public static function get_basic_line_item_array( SI_Invoice $invoice ) {

		$payment_amount = ( si_has_invoice_deposit( $invoice->get_id() ) ) ? $invoice->get_deposit() : $invoice->get_balance();

		$session_line_items   = array();
		$session_line_items[] = array(
			'price_data' => array(
				'currency'     => self::get_currency_code( $invoice->get_id() ),
				'unit_amount'  => self::convert_money_to_cents( $payment_amount, $invoice ),
				'product_data' => array(
					'name'        => sprintf( __( 'Invoice %s', 'sprout-invoices' ), $invoice->get_id() ),
					'description' => html_entity_decode( strip_tags( $invoice->get_title() ), ENT_QUOTES, 'UTF-8' ),
				),
			),
			'quantity'   => 1,
		);

		return $session_line_items;
	}

	/**
	 * Create Deposit Line Item
	 *
	 * @since 20.4
	 * @param  SI_Invoice $invoice Invoice object.
	 * @return array  $session_line_items Stripe line item array.
	 */
	public static function create_deposit_line_item( SI_Invoice $invoice ) {
		$deposit_amount = $invoice->get_deposit();

		if ( 0.00 < $deposit_amount ) {

			$payment_amount = self::convert_money_to_cents( $deposit_amount, $invoice );

			$session_line_items[] = array(
				'price_data' => array(
					'currency'     => self::get_currency_code( $invoice->get_id() ),
					'unit_amount'  => $payment_amount,
					'product_data' => array(
						'name'        => __( 'Deposit', 'sprout-invoices' ),
						'description' => html_entity_decode( strip_tags( $invoice->get_title() ), ENT_QUOTES, 'UTF-8' ),
					),
				),
				'quantity'   => 1,
			);

			return $session_line_items;

		}
	}

	/**
	 * Get the line item array for a Stripe payment.
	 *
	 * @param  SI_Invoice $invoice Invoice object.
	 * @param  boolean    $return_total_tally Return the total tally instead of the line items.
	 * @return mixed      $session_line_items Stripe line item array.
	 */
	public static function get_line_item_array( SI_Invoice $invoice, $return_total_tally = false ) {
		$invoice_id         = $invoice->get_id();
		$session_line_items = array();
		$total_tally        = 0.00;

		// Handle Deposits
		if ( ! SI_Payment::maybe_paid_deposit( $invoice ) && si_has_invoice_deposit( $invoice_id ) ) {
			$session_line_items = self::create_deposit_line_item( $invoice );
			return $session_line_items;
		}

		$discount = $invoice->get_discount_total();
		if ( $discount < 0.00 ) {
			return false;
		}

		$line_items = $invoice->get_line_items();
		foreach ( $line_items as $position => $data ) {
			$children            = si_line_item_get_children( $data['key'], $line_items );
			$is_parent_line_item = ( ! empty( $children ) ) ? true : false;
			if ( $is_parent_line_item ) {
				continue;
			}

			// Caculate Total else caculate Tax.
			$amount = 0.00;
			if ( ! is_numeric( $data['rate'] ) ) {
				$data['rate'] = floatval( $data['rate'] );
			}

			if ( ! is_numeric( $data['qty'] ) ) {
				$data['qty'] = floatval( $data['qty'] );
			}

			if ( isset( $data['tax'] ) && $data['tax'] > 0 ) {
				$amount = ( $data['rate'] * $data['qty'] ) - ( ( $data['rate'] * $data['qty'] ) * ( $data['tax'] / 100) );
			} elseif ( isset( $data['tax_gst'] ) && $data['tax_gst'] > 0 ) {
				$amount = $data['subtotal'];
			} else {
				$amount = $data['rate'] * $data['qty'];
			}

			$item_type = ( isset( $data['type'] ) && '' !== $data['type'] ) ? $data['type'] : SI_Line_Items::get_default_type();

			// line item without tax.
			if ( $data['total'] ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( $amount, $invoice ),
						'product_data' => array(
							'name'        => apply_filters( 'si_stripe_checkout_line_item_name', sprintf( __( '%2$s - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
							'description' => html_entity_decode( strip_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
						),
					),
					'quantity'   => 1,
				);
			}

			// Line Item with Tax.
			if ( isset( $data['tax'] ) && ! empty( $data['tax'] ) ) {
				// If the $data['tax'] is positive number, then it's a discount and we will not add as a line item.
				if ( 0 > $data['tax'] ) {
					$data['tax']          = $data['rate'] * ( abs( $data['tax'] / 100 ) );
					$session_line_items[] = array(
						'price_data' => array(
							'currency'     => self::get_currency_code( $invoice->get_id() ),
							'unit_amount'  => self::convert_money_to_cents( $data['tax'], $invoice ),
							'product_data' => array(
								'name'        => apply_filters( 'si_stripe_checkout_tax_Name', sprintf( __( 'Tax - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
								'description' => html_entity_decode( wp_strip_all_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
							),
						),
						'quantity'   => 1,
					);
				}
			}

			// VAT Adjustment.
			if ( isset( $data['tax_vat'] ) && ! empty( $data['tax_vat'] ) ) {
				$data['tax_vat']      = $amount * ( $data['tax_vat'] / 100 );
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( $data['tax_vat'], $invoice ),
						'product_data' => array(
							'name'        => apply_filters( 'si_stripe_checkout_Vat_Name', sprintf( __( 'Tax Adjustment - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
							'description' => html_entity_decode( wp_strip_all_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
						),
					),
					'quantity'   => 1,
				);
			}

			// GST Adjustment.
			if ( isset( $data['tax_gst'] ) && $data['tax_gst'] > 0 ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( ( $data['tax_gst'] / 100 ) * $amount, $invoice ),
						'product_data' => array(
							'name'        => apply_filters( 'si_stripe_checkout_GST_Name', sprintf( __( 'GST Adjustment - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
							'description' => html_entity_decode( strip_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
						),
					),
					'quantity'   => 1,
				);
			}

			// HST Adjustment.
			if ( isset( $data['tax_hst'] ) && $data['tax_hst'] > 0 ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( ( $data['tax_hst'] / 100 ) * $amount, $invoice ),
						'product_data' => array(
							'name'        => apply_filters( 'si_stripe_checkout_HST_Name', sprintf( __( 'HST Adjustment - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
							'description' => html_entity_decode( strip_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
						),
					),
					'quantity'   => 1,
				);
			}

			// PST Adjustment.
			if ( isset( $data['tax_pst'] ) && $data['tax_pst'] > 0 ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( ( $data['tax_pst'] / 100 ) * $amount, $invoice ),
						'product_data' => array(
							'name'        => apply_filters( 'si_stripe_checkout_PST_Name', sprintf( __( 'PST Adjustment - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
							'description' => html_entity_decode( strip_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
						),
					),
					'quantity'   => 1,
				);
			}
			// IVA Adjustment.
			if ( isset( $data['tax_iva'] ) && $data['tax_iva'] > 0 ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( $data['tax_iva'], $invoice ),
						'product_data' => array(
							'name'        => apply_filters( 'si_stripe_checkout_IVA_Name', sprintf( __( 'IVA Adjustment - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
							'description' => html_entity_decode( strip_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
						),
					),
					'quantity'   => 1,
				);
			}
			// IRPF Adjustment.
			if ( isset( $data['tax_irpf'] ) && '' !== $data['tax_irpf'] ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( $data['tax_irpf'], $invoice ),
						'product_data' => array(
							'name'        => apply_filters( 'si_stripe_checkout_IRPF_Name', sprintf( __( 'IRPF Adjustment - %1$s', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data['desc'], $invoice ),
							'description' => html_entity_decode( strip_tags( $data['desc'] ), ENT_QUOTES, 'UTF-8' ),
						),
					),
					'quantity'   => 1,
				);
			}
		}

		// this is for Default Tax on the invoice.
		$invoice_tax = $invoice->get_tax_total();
		if ( $invoice_tax > 0 ) {
			$session_line_items[] = array(
				'price_data' => array(
					'currency'     => self::get_currency_code( $invoice->get_id() ),
					'unit_amount'  => self::convert_money_to_cents( $invoice_tax, $invoice ),
					'product_data' => array(
						'name'     => apply_filters(
							'si_stripe_checkout_line_fees_adjustment_item_name',
							sprintf(
								__( 'Invoice Tax', 'sprout-invoices' ),
								number_format( (float) $position, 1, '.', '' ),
								ucfirst( $item_type )
							),
								$data,
								$invoice_id
						),
					),
				),
				'quantity'   => 1,
			);
		}

		// this is for Default Tax 2 on the invoice.
		$invoice_tax2 = $invoice->get_tax2_total();
		if ( $invoice_tax2 > 0 ) {
			$session_line_items[] = array(
				'price_data' => array(
					'currency'     => self::get_currency_code( $invoice->get_id() ),
					'unit_amount'  => self::convert_money_to_cents( $invoice_tax2, $invoice ),
					'product_data' => array(
						'name'     => apply_filters(
							'si_stripe_checkout_line_fees_adjustment_item_name',
							sprintf(
								__( 'Invoice Tax 2', 'sprout-invoices' ),
								number_format( (float) $position, 1, '.', '' ),
								ucfirst( $item_type )
							),
								$data,
								$invoice_id
						),
					),
				),
				'quantity'   => 1,
			);
		}

		// this is for Stripe Fee in Settings.
		$fees = $invoice->get_fees_total();
		if ( isset( $fees['payment_service_fee'] ) ) {
			if ( $fees > 0.00 ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( $fees, $invoice ),
						'product_data' => array(
							'name' => apply_filters( 'si_stripe_checkout_line_fees_adjustment_item_name', sprintf( __( 'Stripe Payment Fee', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data, $invoice_id ),
						),
					),
					'quantity'   => 1,
				);

			}
		} elseif ( class_exists( 'SI_Service_Fee' ) ) {
			$stripe_fee_percent = SI_Service_Fee::get_service_fee( 'SI_Stripe_Checkout' );
			$stripe_fee         = $invoice->get_calculated_total( false ) * ( $stripe_fee_percent / 100 );
			if ( $stripe_fee > 0.00 ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( $stripe_fee, $invoice ),
						'product_data' => array(
							'name' => apply_filters( 'si_stripe_checkout_line_fees_adjustment_item_name', sprintf( __( 'Stripe Payment Fee', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data, $invoice_id ),
						),
					),
					'quantity'   => 1,
				);

			}
		} else {
			if ( $fees > 0.00 ) {
				$session_line_items[] = array(
					'price_data' => array(
						'currency'     => self::get_currency_code( $invoice->get_id() ),
						'unit_amount'  => self::convert_money_to_cents( $fees, $invoice ),
						'product_data' => array(
							'name' => apply_filters( 'si_stripe_checkout_line_fees_adjustment_item_name', sprintf( __( 'Additional Invoice Fees', 'sprout-invoices' ), number_format( (float) $position, 1, '.', '' ), ucfirst( $item_type ) ), $data, $invoice_id ),
						),
					),
					'quantity'   => 1,
				);

			}
		}

		return $session_line_items;
	}

	/**
	 * Webhook listener for Stripe Checkout.
	 *
	 * @return false if request is not from stripe.
	 */
	public static function webhook_listener() {
		if ( ! isset( $_REQUEST[ self::API_QUERY_VAR ] ) ) {
			return;
		}

		self::load_sdk();

		$payload = @file_get_contents( 'php://input' );
		$event   = null;

		try {

			$event = \Stripe\Event::constructFrom( json_decode( $payload, true ) );

		} catch ( \UnexpectedValueException $e ) {

			// Invalid payload
			http_response_code( 400 );
			exit();

		}

		// Handle the event
		switch ( $event->type ) {

			case 'checkout.session.completed':

				$session_object = $event->data->object;
				$payment_id     = $session_object->client_reference_id;

				self::maybe_create_recurring_payment( $payment_id, $session_object );

				$success = self::complete_payment( $payment_id, $session_object );

				if ( $success ) {
					http_response_code( 200 );
					exit();
				}

				break;

			default:
				// Unexpected event type
		}
		http_response_code( 400 );
		exit();

	}

	/**
	 * Get webhook url.
	 *
	 * @return string
	 */
	public static function get_webhook_url() {
		return add_query_arg( array( self::API_QUERY_VAR => 'callback' ), home_url() );
	}

	/**
	 * Create webhook endpoint.
	 *
	 * @return void
	 */
	public static function maybe_create_hook_endpoint() {
		self::load_sdk();
		$endpoints = \Stripe\WebhookEndpoint::all( array( 'limit' => 100 ) );

		$out_webhooks_count = 0;
		if ( isset( $endpoints->data ) && ! empty( $endpoints->data ) ) {
			foreach ( $endpoints->data as $key => $endpoint ) {

				if ( self::get_webhook_url() === $endpoint->url ) {

					$out_webhooks_count++;

					if ( $out_webhooks_count > 1 ) {
						$endpoint = \Stripe\WebhookEndpoint::retrieve( $endpoint->id );
						$endpoint->delete();
					}
				}
			}
		}

		if ( $out_webhooks_count < 1 ) {
			$endpoint = \Stripe\WebhookEndpoint::create(
				array(
					'url'            => self::get_webhook_url(),
					'enabled_events' => array( 'checkout.session.completed' ),
				)
			);
		}

	}

	/**
	 * Create the recurring payment profile.
	 */
	private static function create_recurring_payment_plan( SI_Invoice $invoice ) {

		self::load_sdk();

		try {
			$invoice_id = $invoice->get_id();
			$term       = SI_Subscription_Payments::get_term( $invoice_id ); // day, week, month, or year
			$duration   = (int) SI_Subscription_Payments::get_duration( $invoice_id );
			$price      = SI_Subscription_Payments::get_renew_price( $invoice_id );

			$plan_id = $invoice_id . self::convert_money_to_cents( sprintf( '%0.2f', $price ), $invoice );
			$name    = get_the_title( $invoice_id );

			$plan_args = array(
				'product'  => array( 'name' => $name, 'id' => $invoice_id ),
				'nickname' => sprintf( __( 'Invoice: %s', 'sprout-invoices' ), $name ),
				'amount'   => self::convert_money_to_cents( sprintf( '%0.2f', $price ), $invoice ),
				'currency' => self::get_currency_code( $invoice_id ),
				'interval' => $term,
				'id'       => $plan_id,
			);
			// Recurring Plan the customer will be changed to.
			$plan = \Stripe\Plan::create( apply_filters( 'si_stripe_plan_args', $plan_args, $invoice ) );
			do_action( 'si_log', __CLASS__ . '::' . __FUNCTION__ . ' - Stripe PLAN Response', $plan );

			do_action( 'si_stripe_recurring_payment_plan_created', $plan );

			return $plan_id;

		} catch ( Exception $e ) {
			if ( 'resource_already_exists' === $e->jsonBody['error']['code'] ) {
				return $plan_id;
			}

			self::set_error_messages( $e->getMessage() );
			return false;

		}
	}

	/**
	 * Maybe create the recurring payment profile.
	 *
	 * @param  int    $payment_id Payment ID.
	 * @param  object $data       Stripe data.
	 * @return false if not recurring or subscription is not set in stripe.
	 */
	public static function maybe_create_recurring_payment( $payment_id, $data ) {

		$payment = SI_Payment::get_instance( $payment_id );

		if ( ! isset( $payment ) ) {
			return;
		}

		$invoice_id = $payment->get_invoice_id();
		$invoice    = SI_Invoice::get_instance( $invoice_id );
		$user       = si_who_is_paying( $invoice );

		if ( ! si_is_invoice_recurring( $invoice ) ) {
			return;
		}

		if ( ! isset( $data->subscription ) || '' == $data->subscription ) {
			return;
		}

		$subscribe = array(
			'customer_id'     => $data->customer,
			'subscription_id' => $data->subscription,
			'amount'          => $data->display_items[0]->amount,
			'plan'            => $data->display_items[0]->plan->id,
			'email'           => $data->customer_email,
			'data'            => $data,
		);

		// Payment
		$payment_id = SI_Payment::new_payment(
			array(
				'payment_method' => self::PAYMENT_METHOD,
				'invoice'        => $invoice_id,
				'amount'         => $data->display_items[0]->amount,
				'data'           => array(
					'live'         => ( self::MODE_LIVE === self::$api_mode ),
					'api_response' => $subscribe,
				),
			),
			SI_Payment::STATUS_RECURRING
		);

		do_action( 'si_stripe_recurring_payment_profile_created', $payment_id );

	}



	public function verify_recurring_payment( SI_Payment $payment ) {
		$invoice_id = $payment->get_invoice_id();
		if ( ! $invoice_id ) {
			return;
		}
		$invoice = SI_Invoice::get_instance( $invoice_id );

		$data = $payment->get_data();
		if ( ! isset( $data['api_response']['customer_id'] ) || ! isset( $data['api_response']['subscription_id'] ) ) {
			return false;
		}
		$status = self::get_subscription_status( $data['api_response']['customer_id'], $data['api_response']['subscription_id'] );

		do_action( 'si_verify_recurring_payment_status', $status, $payment );

		if ( 'active' != $status ) {
			$payment->set_status( SI_Payment::STATUS_CANCELLED );
		}

		return $status;
	}

	public static function get_subscription_status( $customer_id, $subscription_id ) {
		$status = false;
		try {
			self::load_sdk();
			$customer = \Stripe\Customer::retrieve( $customer_id );
			if ( isset( $customer->subscriptions ) ) {
				$subscription = $customer->subscriptions->retrieve( $subscription_id );
				$status       = $subscription->status;
			}
		} catch ( Exception $e ) {
			// do nothing
		}
		return $status;
	}

	public function cancel_recurring_payment( SI_Invoice $invoice ) {
		$payment = self::get_recurring_payment( $invoice );
		if ( ! $payment ) {
			return;
		}
		$data = $payment->get_data();
		if ( ! isset( $data['api_response']['subscription_id'] ) || ! isset( $data['api_response']['customer_id'] ) ) {
			return false;
		}
		try {
			self::load_sdk();
			$customer     = \Stripe\Customer::retrieve( $data['api_response']['customer_id'] );
			$subscription = $customer->subscriptions->retrieve( $data['api_response']['subscription_id'] );
			$subscription->cancel();
		} catch ( Exception $e ) {
			$status = false;
		}
		do_action( 'si_cancelled_recurring_payment', $subscription, $invoice );
	}

	public static function stripe_profile_link( $payment ) {
		if ( $payment->get_payment_method() !== self::PAYMENT_METHOD ) {
			return;
		}
		$data = $payment->get_data();

		if ( isset( $data['api_response']['subscription_id'] ) && isset( $data['api_response']['customer_id'] ) ) {

			$status = self::get_subscription_status( $data['api_response']['customer_id'], $data['api_response']['subscription_id'] );

			printf( __( '<b>Current Payment Status:</b> <code>%s</code>', 'sprout-invoices' ), $status );
			echo ' &mdash; ';
			esc_html_e( 'Stripe Subscription ID: ' , 'sprout-invoices' );
			if ( isset( $data['live'] ) && ! $data['live'] ) {
				printf( '<a class="payment_profile_link" href="https://dashboard.stripe.com/test/customers/%s" target="_blank">%s</a>', $data['api_response']['customer_id'], $data['api_response']['subscription_id'] );
			} else {
				printf( '<a class="payment_profile_link" href="https://dashboard.stripe.com/customers/%s" target="_blank">%s</a>', $data['api_response']['customer_id'], $data['api_response']['subscription_id'] );
			}
		}
	}

	/**
	 * Check for discounts and previous payments.
	 * Previous payments include deposits, admin payments, and partial payments.
	 * The total amount returned will be added as a coupon in Stripe.
	 *
	 * @since 20.4
	 * @param SI_Invoice $invoice The invoice object.
	 * @return float $total the total amount to add to coupon.
	 */
	public static function calculate_coupon_total( SI_Invoice $invoice ) {
		// Previous Payments.
		$payments_total          = $invoice->get_payments_total( false );
		$previous_payments_total = 0;
		if ( $payments_total > 0 ) {
			$payments = $invoice->get_payments();
			$payment  = array();
			foreach ( $payments as $payments_id ) {
				$payment_object = SI_Payment::get_instance( $payments_id );
				if ( $payment_object->get_status() !== SI_Payment::STATUS_VOID ) {
					$previous_payments_total += $payment_object->get_amount();
				}
			}
		}

		// Discounts.
		$discount_total = $invoice->get_discount_total();

		$total = $previous_payments_total + $discount_total;

		return $total;
	}

	/**
	 * Convert money to cents.
	 *
	 * @param  float      $value   The value to convert.
	 * @param  SI_Invoice $invoice The invoice object.
	 * @return float
	 */
	private static function convert_money_to_cents( $value, $invoice ) {
		if ( ! apply_filters( 'si_convert_money_to_cents_for_stripe', true, $invoice ) ) {
			return ( float ) $value;
		}
		// strip out commas
		$value = preg_replace( '/\,/i', '', $value );
		// strip out all but numbers, dash, and dot
		$value = preg_replace( '/([^0-9\.\-])/i', '', $value );
		// make sure we are dealing with a proper number now, no +.4393 or 3...304 or 76.5895,94
		if ( ! is_numeric( $value ) ) {
			return 0.00;
		}
		// convert to a float explicitly
		$value = ( float ) $value;
		return round( $value, 2 ) * 100;
	}

	private static function convert_cents_to_money( $value, $invoice ) {
		if ( ! apply_filters( 'si_convert_money_to_cents_for_stripe', true, $invoice ) ) {
			return ( float ) $value;
		}
		// strip out commas
		$value = preg_replace( '/\,/i', '', $value );
		// strip out all but numbers, dash, and dot
		$value = preg_replace( '/([^0-9\.\-])/i', '', $value );
		// make sure we are dealing with a proper number now, no +.4393 or 3...304 or 76.5895,94
		if ( ! is_numeric( $value ) ) {
			return 0.00;
		}
		// convert to a float explicitly
		return number_format( floatval( $value / 100 ), 2 );
	}


	private static function get_currency_code( $invoice_id ) {
		return apply_filters( 'si_currency_code', self::$currency_code, $invoice_id, self::PAYMENT_METHOD );
	}

	/**
	 * Grabs error messages from a response and displays them to the user
	 *
	 * @param array   $response
	 * @param bool    $display
	 * @return void
	 */
	private static function set_error_messages( $message, $display = true ) {
		if ( $display ) {
			self::set_message( $message, self::MESSAGE_STATUS_ERROR );
		} else {
			do_action( 'si_error', __CLASS__ . '::' . __FUNCTION__ . ' - error message from stripe', $message );
		}
	}

	public static function load_addon_view( $view, $args, $allow_theme_override = true ) {
		add_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		$view = self::load_view( $view, $args, $allow_theme_override );
		remove_filter( 'si_views_path', array( __CLASS__, 'addons_view_path' ) );
		return $view;
	}

	public static function load_addon_view_to_string( $view, $args, $allow_theme_override = true ) {
		ob_start();
		self::load_addon_view( $view, $args, $allow_theme_override );
		return ob_get_clean();
	}

	public static function addons_view_path() {
		return SA_ADDON_STRIPE_CHECKOUT_PATH . '/views/';
	}
}
SI_Stripe_Checkout::register();
